// ==============================================================
//
// delta2_core
// Control module for delta2_core vessel class
//
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"
#include <cstdio>
#include <cmath>
#include <string>
#include <stdio.h>

// ==============================================================
// Some vessel parameters
// ==============================================================


const double STG1DRYMASS = 5680;



//#define GREEN RGB(125, 255, 125)

void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*22.5;
	static const double istep = 1.0/step;
	static const int nabsc = 17;
	static const double CL[nabsc] = {0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0};


	aoa += PI;
	int idx = max (0, min (15, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = 0.0;
	*cd = 0.11 + oapiGetInducedDrag (*cl, 0.16, 0.2);
	
	
}

// 2. horizontal lift component (vertical stabiliser and body)

void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*22.5;
	static const double istep = 1.0/step;
	static const int nabsc = 17;
	static const double CL[nabsc] = {0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0};

	beta += PI;
	int idx = max (0, min (15, (int)(beta*istep)));
	double d = beta*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = 0.0;
	*cd = 0.11 + oapiGetInducedDrag (*cl, 0.16, 0.2);
	
}

// ==============================================================
// Shuttle-PB class interface
// ==============================================================

class delta2_core: public VESSEL3 {
public:
	delta2_core (OBJHANDLE hVessel, int flightmodel);
	~delta2_core ();
	void clbkSetClassCaps (FILEHANDLE cfg);
	//void clbkPostCreation();
	void clbkPreStep(double SimT, double SimDT, double MJD);
	//void clbkPostStep (double simt, double simdt, double mjd);
	void clbkSaveState (FILEHANDLE scn);
	void clbkLoadStateEx (FILEHANDLE scn, void *status);
	//int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);
	//bool clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp);
	//bool clbkLoadGenericCockpit ();
	
private:

	double timer;
	
};

delta2_core::delta2_core (OBJHANDLE hVessel, int flightmodel)
: VESSEL3 (hVessel, flightmodel)
{
    

}

delta2_core::~delta2_core ()
{

}


// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void delta2_core::clbkSetClassCaps (FILEHANDLE cfg)
{
	
	timer = 0; // self delete timer
	
	// physical vessel parameters
	SetEmptyMass (STG1DRYMASS);
	SetSize (10);
	SetPMI(_V(9,9,1.9));
	SetCrossSections (_V(22,22,4));
	SetTouchdownPoints ( _V(0,-5, -10), _V(-5, 3, -10), _V( 5, 3,-10));
	
	SetCW(0.5,1,1,1);
	SetRotDrag (_V(0.5,0.5,0.1));

	// camera parameters
	SetCameraOffset (_V(0,2,-5));
	SetCameraRotationRange(PI,PI,(0.4*PI),(0.4*PI));


	SetMeshVisibilityMode(AddMesh("Delta2/delta2core"),MESHVIS_ALWAYS);


	

	CreateAirfoil (LIFT_VERTICAL, _V(0,0,0), VLiftCoeff, 5, 10, 0.16);
	CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,0), HLiftCoeff, 5, 10, 0.16);

}

//void delta2_core::clbkPostCreation ()
//{
//
//
//}


// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new delta2_core (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (delta2_core*)vessel;
}


void delta2_core::clbkSaveState (FILEHANDLE scn)
{
	char cbuf[256];

	// default vessel parameters
	VESSEL3::clbkSaveState(scn);

	sprintf (cbuf, "%0.4f", timer);
	oapiWriteScenario_string (scn, "ANMS", cbuf);



}

void delta2_core::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
    char *line;
	while (oapiReadScenario_nextline (scn, line)) {
		if (!strnicmp (line, "ANMS", 4)) {
		sscanf (line+4, "%lf", &timer);
		} else {
            ParseScenarioLineEx (line, vs);
			// unrecognised option - pass to Orbiter's generic parser
        }
   }

	

}



void delta2_core::clbkPreStep(double SimT, double SimDT, double MJD)
{
	timer = timer + SimDT;
	if(timer > 600)oapiDeleteVessel(GetHandle());

}

