// ==============================================================
//
// delta2_stg2
// Control module for delta2_stg2 vessel class
//
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"
#include <cstdio>
#include <cmath>
#include <string>
#include <stdio.h>

// ==============================================================
// Some vessel parameters
// ==============================================================


const double STG2ISP0 = 3443;//3130;
const double STG2ISPV = 3443;//3130;
const double STG2MAINTH0 = 43655;
const double STG2MAINTH1 = 43655;
const double STG2RCSTH1 = 200; // racs
const double STG2RCSTH2 = 2000; // thrust vector
const double STG2FUELMASS = 6000;
const double STG2DRYMASS = 900;
const double STG2RCSFUELMASS = 50;
const double STG2RCSISP = 800;



//#define GREEN RGB(125, 255, 125)

void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*22.5;
	static const double istep = 1.0/step;
	static const int nabsc = 17;
	static const double CL[nabsc] = {0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0};


	aoa += PI;
	int idx = max (0, min (15, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = 0.0;
	*cd = 0.11 + oapiGetInducedDrag (*cl, 0.16, 0.2);
	
	
}

// 2. horizontal lift component (vertical stabiliser and body)

void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*22.5;
	static const double istep = 1.0/step;
	static const int nabsc = 17;
	static const double CL[nabsc] = {0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0};

	beta += PI;
	int idx = max (0, min (15, (int)(beta*istep)));
	double d = beta*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = 0.0;
	*cd = 0.11 + oapiGetInducedDrag (*cl, 0.16, 0.2);
	
}

// ==============================================================
// Shuttle-PB class interface
// ==============================================================

class delta2_stg2: public VESSEL3 {
public:
	delta2_stg2 (OBJHANDLE hVessel, int flightmodel);
	~delta2_stg2 ();
	void clbkSetClassCaps (FILEHANDLE cfg);
	//void clbkPostCreation();
	void clbkPreStep(double SimT, double SimDT, double MJD);
	//void clbkPostStep (double simt, double simdt, double mjd);
	void clbkSaveState (FILEHANDLE scn);
	void clbkLoadStateEx (FILEHANDLE scn, void *status);
	//int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);
	//bool clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp);
	//bool clbkLoadGenericCockpit ();
	
private:
	
	THRUSTER_HANDLE th_main[1],th_rcs[12], th_group[2];
	PROPELLANT_HANDLE hProp2, hRCSProp2;
	THGROUP_HANDLE htgrppup2,htgrppdown2, htgrpyright2, htgrpyleft2;
	MGROUP_TRANSFORM *spin1;
	UINT anim_spin;


	void DefineAnimations (void);
	void ConfigRCS2(void);
	void RevertRCS2(void);


	double timer;
	int rcs_sw2;

	double spin_proc, drot;
	int spin_status;
	
};

delta2_stg2::delta2_stg2 (OBJHANDLE hVessel, int flightmodel)
: VESSEL3 (hVessel, flightmodel)
{
    

}

delta2_stg2::~delta2_stg2 ()
{
	delete spin1;
}

void delta2_stg2::DefineAnimations (void)
{
// 1. SPIN TABLE
	anim_spin = CreateAnimation (0);
	static UINT SpinGrp1[3] = {0,2,3};
	spin1 = new MGROUP_ROTATE  (0, SpinGrp1, 3, _V(0,0,0), _V(0,0,-1), (float)(360*RAD));
	AddAnimationComponent (anim_spin, 0, 1, spin1);


}
// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void delta2_stg2::clbkSetClassCaps (FILEHANDLE cfg)
{
	rcs_sw2 = 0; // 2nd stage rcs : thrust vector or cold gas 
	timer = 0; // self delete timer
	spin_proc = 0; // spin animation
	drot = 0.25;
	spin_status = 0;
	
	// physical vessel parameters
	SetEmptyMass (STG2DRYMASS);
	SetSize (5);
	SetPMI(_V(9,9,1.9));
	SetCrossSections (_V(22,22,4));
	SetTouchdownPoints ( _V(0,-5, -10), _V(-5, 3, -10), _V( 5, 3,-10));
	SetCameraOffset (_V(0,2,0));
	
	SetCW(0.5,1,1,1);
	SetRotDrag (_V(0.5,0.5,0.1));

	// camera parameters
	SetCameraOffset (_V(0,2,-5));
	SetCameraRotationRange(PI,PI,(0.4*PI),(0.4*PI));


	SetMeshVisibilityMode(AddMesh("Delta2/delta2stg2"),MESHVIS_ALWAYS);


	DefineAnimations();

	// propellant resources
	
	
	hProp2 = CreatePropellantResource (STG2FUELMASS,STG2FUELMASS);
	hRCSProp2 = CreatePropellantResource (STG2RCSFUELMASS,STG2RCSFUELMASS);
	// ***************** thruster definitions *******************

	th_main[0] = CreateThruster (_V( 0, 0, -3), _V( 0,0,1), STG2MAINTH1, hProp2, STG2ISPV, STG2ISP0);
	
	
	

	////// stg2

	th_rcs[0] = CreateThruster (_V( 0, 0, -3), _V( 1,0,0), STG2RCSTH2, hProp2, STG2ISPV, STG2ISP0); //yl
	th_rcs[1] = CreateThruster (_V( 0, 0, -3), _V(-1,0,0), STG2RCSTH2, hProp2, STG2ISPV, STG2ISP0); //  yr
	th_rcs[2] = CreateThruster (_V( 0, 0, -3), _V( 0,-1,0), STG2RCSTH2, hProp2, STG2ISPV, STG2ISP0); // pup
	th_rcs[3] = CreateThruster (_V( 0, 0, -3), _V( 0,1,0), STG2RCSTH2, hProp2, STG2ISPV, STG2ISP0); // pd

	th_rcs[4] = CreateThruster (_V(0.077,0.73,-0.398), _V(-1, 0, 0), STG2RCSTH1, hRCSProp2, STG2RCSISP, STG2RCSISP); // bl/yr
	th_rcs[5] = CreateThruster (_V(-0.077,-0.73,-0.398), _V( 1, 0, 0), STG2RCSTH1, hRCSProp2, STG2RCSISP, STG2RCSISP);// bl/yl
	th_rcs[6] = CreateThruster (_V(0.077,-0.73,-0.398), _V(-1, 0, 0), STG2RCSTH1, hRCSProp2, STG2RCSISP, STG2RCSISP); // br/yr
	th_rcs[7] = CreateThruster (_V(-0.077,0.73,-0.398), _V( 1, 0, 0), STG2RCSTH1, hRCSProp2, STG2RCSISP, STG2RCSISP);// br/yl

	th_rcs[8] = CreateThruster (_V(0,0.79,-0.398), _V( 0,-1, 0), STG2RCSTH1, hRCSProp2, STG2RCSISP, STG2RCSISP); // pu
	th_rcs[9] = CreateThruster (_V(0,0.79,-0.441), _V( 0,-1, 0), STG2RCSTH1, hRCSProp2, STG2RCSISP, STG2RCSISP);// pu
	th_rcs[10] = CreateThruster (_V(0,-0.79,-0.398), _V( 0, 1, 0), STG2RCSTH1, hRCSProp2, STG2RCSISP, STG2RCSISP); // pd
	th_rcs[11] = CreateThruster (_V(0,-0.79,-0.441), _V( 0, 1, 0), STG2RCSTH1, hRCSProp2, STG2RCSISP, STG2RCSISP);// pd


	th_group[0] = th_rcs[2];
	htgrppup2 = CreateThrusterGroup (th_group, 1, THGROUP_ATT_PITCHUP);

	th_group[0] = th_rcs[3];
	htgrppdown2 = CreateThrusterGroup (th_group, 1, THGROUP_ATT_PITCHDOWN);

	th_group[0] = th_rcs[1];
	htgrpyright2 = CreateThrusterGroup (th_group, 1, THGROUP_ATT_YAWRIGHT);

	th_group[0] = th_rcs[0];
	htgrpyleft2 = CreateThrusterGroup (th_group, 1, THGROUP_ATT_YAWLEFT);

	th_group[0] = th_rcs[6];
	th_group[1] = th_rcs[7];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_BANKRIGHT);

	th_group[0] = th_rcs[4];
	th_group[1] = th_rcs[5];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_BANKLEFT);


	th_group[0] = th_main[0];
	CreateThrusterGroup (th_group, 1, THGROUP_MAIN);

	SURFHANDLE exhaust = oapiRegisterExhaustTexture ("Exhaust2");
	SURFHANDLE exhaustrcs = oapiRegisterExhaustTexture ("exhaust_atrcs");

	AddExhaust(th_main[0],15,1.25,_V(0,0,-3.3),_V(0,0,-1),exhaust);


	double exlenrcs = 1;
	double exwidrcs = 0.1;

	AddExhaust(th_rcs[4],exlenrcs,exwidrcs,exhaustrcs);
	AddExhaust(th_rcs[5],exlenrcs,exwidrcs,exhaustrcs);
	AddExhaust(th_rcs[6],exlenrcs,exwidrcs,exhaustrcs);
	AddExhaust(th_rcs[7],exlenrcs,exwidrcs,exhaustrcs);

	AddExhaust(th_rcs[8],exlenrcs,exwidrcs,exhaustrcs);
	AddExhaust(th_rcs[9],exlenrcs,exwidrcs,exhaustrcs);
	AddExhaust(th_rcs[10],exlenrcs,exwidrcs,exhaustrcs);
	AddExhaust(th_rcs[11],exlenrcs,exwidrcs,exhaustrcs);

	///////////////////////////////////////////////////////
	/////AirFoils

	CreateAirfoil (LIFT_VERTICAL, _V(0,0,0), VLiftCoeff, 5, 10, 0.16);
	CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,0), HLiftCoeff, 5, 10, 0.16);

}

//void delta2_stg2::clbkPostCreation ()
//{
//
//
//}


// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new delta2_stg2 (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (delta2_stg2*)vessel;
}


void delta2_stg2::clbkSaveState (FILEHANDLE scn)
{
	char cbuf[256];

	// default vessel parameters
	VESSEL3::clbkSaveState(scn);

	sprintf (cbuf, "%d %0.4f %0.4f %0.4f",spin_status, spin_proc, timer, drot);
	oapiWriteScenario_string (scn, "ANMS", cbuf);



}

void delta2_stg2::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
    char *line;
	while (oapiReadScenario_nextline (scn, line)) {
		if (!strnicmp (line, "ANMS", 4)) {
		sscanf (line+4, "%d%lf%lf%lf", &spin_status, &spin_proc, &timer, &drot);
		} else {
            ParseScenarioLineEx (line, vs);
			// unrecognised option - pass to Orbiter's generic parser
        }
   }

	

}



void delta2_stg2::clbkPreStep(double SimT, double SimDT, double MJD)
{
	timer = timer + SimDT;
	if(timer > 1000)oapiDeleteVessel(GetHandle());


double mth_lev = GetThrusterLevel(th_main[0]);

if (mth_lev == 0 && rcs_sw2 == 0) ConfigRCS2();
else if (mth_lev > 0 && rcs_sw2 == 1) RevertRCS2();


////////// animations ////////////////////////////

if (spin_status == 0)
{
	drot = drot - (0.125*0.25*SimDT);
	if(drot < 0)
	{
		drot = 0;
		spin_status = 1;
	}
	spin_proc = spin_proc + (drot*SimDT);
	if(spin_proc > 1) spin_proc = spin_proc - ((int)(spin_proc));
}
SetAnimation(anim_spin, spin_proc);

}


void delta2_stg2::RevertRCS2 (void)
{
	SetThrusterGroupLevel(htgrppup2,0);
	SetThrusterGroupLevel(htgrppdown2,0);
	SetThrusterGroupLevel(htgrpyright2,0);
	SetThrusterGroupLevel(htgrpyleft2,0);

	DelThrusterGroup(htgrppup2,THGROUP_ATT_PITCHUP, false);
	DelThrusterGroup(htgrppdown2,THGROUP_ATT_PITCHDOWN, false);
	DelThrusterGroup(htgrpyright2,THGROUP_ATT_YAWRIGHT, false);
	DelThrusterGroup(htgrpyleft2,THGROUP_ATT_YAWLEFT, false);

	th_group[0] = th_rcs[2];
	htgrppup2 = CreateThrusterGroup (th_group, 1, THGROUP_ATT_PITCHUP);

	th_group[0] = th_rcs[3];
	htgrppdown2 = CreateThrusterGroup (th_group, 1, THGROUP_ATT_PITCHDOWN);

	th_group[0] = th_rcs[1];
	htgrpyright2 = CreateThrusterGroup (th_group, 1, THGROUP_ATT_YAWRIGHT);

	th_group[0] = th_rcs[0];
	htgrpyleft2 = CreateThrusterGroup (th_group, 1, THGROUP_ATT_YAWLEFT);

	rcs_sw2 = 0;
}

void delta2_stg2::ConfigRCS2 (void)
{
	SetThrusterGroupLevel(htgrppup2,0);
	SetThrusterGroupLevel(htgrppdown2,0);
	SetThrusterGroupLevel(htgrpyright2,0);
	SetThrusterGroupLevel(htgrpyleft2,0);

	DelThrusterGroup(htgrppup2,THGROUP_ATT_PITCHUP, false);
	DelThrusterGroup(htgrppdown2,THGROUP_ATT_PITCHDOWN, false);
	DelThrusterGroup(htgrpyright2,THGROUP_ATT_YAWRIGHT, false);
	DelThrusterGroup(htgrpyleft2,THGROUP_ATT_YAWLEFT, false);

	th_group[0] = th_rcs[8];
	th_group[1] = th_rcs[9];
	htgrppup2 = CreateThrusterGroup (th_group, 2, THGROUP_ATT_PITCHUP);

	th_group[0] = th_rcs[10];
	th_group[1] = th_rcs[11];
	htgrppdown2 = CreateThrusterGroup (th_group, 2, THGROUP_ATT_PITCHDOWN);

	th_group[0] = th_rcs[4];
	th_group[1] = th_rcs[6];
	htgrpyright2 = CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWRIGHT);

	th_group[0] = th_rcs[5];
	th_group[1] = th_rcs[7];
	htgrpyleft2 = CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWLEFT);

	rcs_sw2 = 1;
}


//bool delta2_stg2::clbkLoadGenericCockpit ()
//{
//SetCamDir();
//return true;
//}

//void delta2_stg2::SetCamDir()
//{
//	VECTOR3 camdefdir = _V(0,0,1);
//	if(cam_status == 1)camdefdir = _V(0,0,-1);
//	SetCameraDefaultDirection(camdefdir, 0);
//	oapiCameraAttach(GetHandle(), 2);
//}

