// ==============================================================
//
// delta7926
// Control module for delta7926 vessel class
//
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"
#include "OrbiterSoundSDK40.h"
#include <cstdio>
#include <cmath>
#include <string>
#include <stdio.h>
#include "gcAPI.h"

// ==============================================================
// Some vessel parameters
// ==============================================================

#define CTDN	1
#define SEP		2

const DWORD COMM_SET_PLD = 2901;
const DWORD COMM_SET_APO = 2902;
const DWORD COMM_SET_PER = 2903;
const DWORD COMM_SET_INC = 2904;
const DWORD COMM_SEND_MAIN = 2905;
const DWORD COMM_SEND_RCS = 2906;

const double STG1ISP0 = 2695;//2495;
const double STG1ISPV = 2965;
const double STG1MAINTH0 = 890000;
const double STG1MAINTH1 = 1055000;
const double STG1RCSTH = 10000; // thrust vector
const double STG1DRYMASS = 5680;
const double STG1FUELMASS = 96120;

const double STG2ISP0 = 3443;//3130;
const double STG2ISPV = 3443;//3130;
const double STG2MAINTH0 = 43655;
const double STG2MAINTH1 = 43655;
const double STG2RCSTH1 = 200; // racs
const double STG2RCSTH2 = 2000; // thrust vector
const double STG2FUELMASS = 6000;
const double STG2DRYMASS = 900;
const double STG2RCSFUELMASS = 50;
const double STG2RCSISP = 800;

const double STG3DRYMASS = 82;
const double STG3FUELMASS = 1067;
const double STG3ISP0 = 2864;
const double STG3MAINTH0 = 48506;

const VECTOR3 PLD_ATT =_V( 0, 0, 4.37);
const VECTOR3 STG1_OFF = _V(0,0,-14.8);
const VECTOR3 STG3_OFF = _V(0,0,3.78);
const VECTOR3 YOYOS_OFF = _V(0, 0, 4.1);
const VECTOR3 YOYO1_OFF = _V(0, 4, 0.32);
const VECTOR3 YOYO2_OFF = _V(0,-4, 0.32);
const VECTOR3 FAIR_OFF = _V(0,0,5);
const VECTOR3 FAIR_OFF2 = _V(0,0,5.35);

const double SRB_DRYMASS = 1102*3;
const double SRB_FUELMASS = 11766*3;
const double SRB_ISP0 = 2272.5;//2405;
const double SRB_ISPV = 2539.5;//2688;
const double SRB_THRUST0 = 450000*3;
const double SRB_THRUSTV = 460350*3;

const VECTOR3 SRB1_OFF = _V( 1.7, 0,-22.2);



const VECTOR3 ADAPTOR_OFF = _V(0,0,4.37);



const double mu = 398600439968871.2;
const double rt = 6371010;


#define GREEN RGB(125, 255, 125)

void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*22.5;
	static const double istep = 1.0/step;
	static const int nabsc = 17;
	static const double CL[nabsc] = {0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0};


	aoa += PI;
	int idx = max (0, min (15, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = 0.0;
	*cd = 0.11 + oapiGetInducedDrag (*cl, 0.16, 0.2);
	
	
}

// 2. horizontal lift component (vertical stabiliser and body)

void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*22.5;
	static const double istep = 1.0/step;
	static const int nabsc = 17;
	static const double CL[nabsc] = {0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0};

	beta += PI;
	int idx = max (0, min (15, (int)(beta*istep)));
	double d = beta*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = 0.0;
	*cd = 0.11 + oapiGetInducedDrag (*cl, 0.16, 0.2);
	
}

// ==============================================================
// Shuttle-PB class interface
// ==============================================================

class delta7926: public VESSEL3 {
public:
	delta7926 (OBJHANDLE hVessel, int flightmodel);
	~delta7926 ();
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPostCreation();
	void clbkPreStep(double SimT, double SimDT, double MJD);
	void clbkPostStep (double simt, double simdt, double mjd);
	void clbkSaveState (FILEHANDLE scn);
	void clbkLoadStateEx (FILEHANDLE scn, void *status);
	int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);
	int MyID;	// this is your unique Id, it will identify your ship in OrbiterSound.
	bool clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp);
	bool clbkLoadGenericCockpit ();
	
private:
	char pldname1[256];
	char pldname2[256];
	char pldname3[256];
	int pld_att_status;
	int calc_sw;
	ATTACHMENTHANDLE pld_attach[3];
	ATTACHMENTHANDLE lpd_attach;
	double tgt_inc;
	double ap_tgt;
	double op_meco_alt;
	double op_tgt;
	OBJHANDLE hGref;
	double  FAIR_MASS;
	char adname [256];

	THRUSTER_HANDLE th_srb[3], th_spin[8];
	THRUSTER_HANDLE th_main[3],th_rcs[24], th_group[9];
	THGROUP_HANDLE htgrpmain;
	PROPELLANT_HANDLE hProp1, hProp2, hProp3, hRCSProp2, hPropSRB0, hPropSRB1, hPropSRB2;
	THGROUP_HANDLE htgrppup2,htgrppdown2,htgrpbright2, htgrpbleft2, htgrpyright2, htgrpyleft2;
	VECTOR3 posth[8], dirth[8];
	MGROUP_TRANSFORM *yoyo1_trans, *yoyo2_trans, *yoyo_scale;
	MGROUP_TRANSFORM *spin1, *spin2, *spin3, *spin4;
	UINT anim_spin, anim_yoyo;
	MESHHANDLE hAdaptor;
	PointLight *eng_light;
	ELEMENTS el;
	ORBITPARAM prm;

	OBJHANDLE hpd;

	void DefineAnimations (void);
	void JettisonPld(void);
	void JettisonFair(void);
	void JettisonStg(void);
	void JettisonStg2(void);
	void JettisonSRB0(void);
	void JettisonSRB1(void);
	void JettisonSRB2(void);
	void JettisonYoyos(void);
	void ConfigMain(void);
	void ConfigMain2(void);
	void ConfigRCS2(void);
	void RevertRCS2(void);
	void CalcEmptyMass(void);
	void DoAscent(void);
	void DoOrbit(void);
	double CalcAzimuth();
	void CalcFlightVector();
	void GoToFlightVector();
	double FinalV();
	double GetOS();
	void RollToAz();
	void SetCamDir();
	void KillAllRCS2();
	void SetSpinParams();


	double step_time, old_ap_dist, pitch_angle, pld_mass1, pld_mass2, pld_mass3,timer, vrot_lim, pst_lvl;
	int op_status, ap_status, fair_status, eng_status, rcs_sw2;
	int ap_function, cam_status;
	int start_sw, tu_sw, srb_status;

	double azimuth;	
	VECTOR3 azim_local, vazim1, vazim2, flight_vector;
	double mth_lev;
	bool large_fair;
	bool has_adaptor;

	double spin_proc, yoyo_proc, drot;
	int spin_status, yoyo_status;
	
};

delta7926::delta7926 (OBJHANDLE hVessel, int flightmodel)
: VESSEL3 (hVessel, flightmodel)
{
    

}

delta7926::~delta7926 ()
{
	delete spin1;
	delete spin2;
	delete spin3;
	delete spin4;
	delete yoyo1_trans;
	delete yoyo2_trans;
	delete yoyo_scale;
}

void delta7926::DefineAnimations (void)
{
// 1. SPIN TABLE
	anim_spin = CreateAnimation (0);
	static UINT SpinGrp1[3] = {0,2,3};
	spin1 = new MGROUP_ROTATE  (0, SpinGrp1, 3, _V(0,0,0), _V(0,0,-1), (float)(360*RAD));
	AddAnimationComponent (anim_spin, 0, 1, spin1);

	static UINT SpinGrp2[9] = {0,1,2,3,4,5,6,7,8}; // star37fm
	spin2 = new MGROUP_ROTATE  (13, SpinGrp2, 9, _V(0,0,0), _V(0,0,-1), (float)(360*RAD));
	AddAnimationComponent (anim_spin, 0, 1, spin2);

	static UINT SpinGrp3[3] = {0,1,2}; // yoyos
	spin3 = new MGROUP_ROTATE  (14, SpinGrp3, 3, _V(0,0,0), _V(0,0,-1), (float)(360*RAD));
	AddAnimationComponent (anim_spin, 0, 1, spin3);

	if(has_adaptor == true)
	{
	int groupcount = oapiMeshGroupCount(hAdaptor);
	static UINT SpinGrp4[10] = {0,1,2,3,4,5,6,7,8,9}; // custom adaptor
	spin4 = new MGROUP_ROTATE  (15, SpinGrp4, groupcount, _V(0,0,0), _V(0,0,-1), (float)(360*RAD));
	AddAnimationComponent (anim_spin, 0, 1, spin4);
	}

	// yoyo deploy
	anim_yoyo = CreateAnimation (0);
	static UINT YoyoGrp1[1] = {1};
	yoyo_scale = new MGROUP_SCALE  (14, YoyoGrp1, 1, _V(0,0,0),_V(1,7.752,1));
	AddAnimationComponent (anim_yoyo, 0, 1, yoyo_scale);

	static UINT YoyoGrp2[1] = {2};
	yoyo1_trans = new MGROUP_TRANSLATE  (14, YoyoGrp2, 1, _V(0,3.484,0));
	AddAnimationComponent (anim_yoyo, 0, 1, yoyo1_trans);

	static UINT YoyoGrp3[1] = {0};
	yoyo2_trans = new MGROUP_TRANSLATE  (14, YoyoGrp3, 1, _V(0,-3.484,0));
	AddAnimationComponent (anim_yoyo, 0, 1, yoyo2_trans);

}
// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void delta7926::clbkSetClassCaps (FILEHANDLE cfg)
{
	strcpy(pldname1, "NULL");
	strcpy(pldname2, "NULL");
	strcpy(pldname3, "NULL");
	FAIR_MASS = 880;
	step_time = 0;
	ap_status = 0; // 1st stage ascent autopilot
	op_status = 0; // 2nd stage orbit insertion autopilot
	cam_status = 0; // forward / back view
	ap_function = 0; // ascent autopilot subroutines
	old_ap_dist = 0; // user input update
	pld_mass1 = 0; // payload masses
	pld_mass2 = 0;
	pld_mass3 = 0;
	pitch_angle = 90*RAD; // guidance
	fair_status = 0; // fairing attached / off
	eng_status = 0; // stage 1 /2
	srb_status = 0; // srb attached/finished
	rcs_sw2 = 0; // 2nd stage rcs : thrust vector or cold gas 
	vrot_lim = 5*RAD; // rot speed limit
	azimuth = 0; // autopilot heading
	azim_local = _V(0,-1,0); // autopilot calcs
	vazim1 = _V(0,1,0); // autopilot calcs
	vazim2 = _V(0,1,0); // autopilot calcs
	flight_vector = _V(0,1,0); // autopilot calcs
	timer = 0; // engine/fairing timer
	start_sw = 0; // engine timer
	pst_lvl = 0; // max q pstream
	tu_sw = 0; // liftoff throttle for lpad / no lpad
	mth_lev = 0; // main thrust level
	large_fair = false;// fairing type
	has_adaptor = false; // custom adaptor

	pld_att_status = 0; // payload count
	calc_sw = 0; // empty mass update switch
	tgt_inc = 0; // tgt inclination
	ap_tgt = 186000; // apa at 1st stage meco
	op_meco_alt = 200000; // tgt PeA and seco
	op_tgt = 200000; // tgt ApA

	gcInitialize();

	spin_proc = 0; // spin animation
	drot = 0;
	spin_status = 0;
	yoyo_proc = 0;
	yoyo_status = 0;
	// physical vessel parameters
	SetSize (20);
	SetEmptyMass (STG1DRYMASS+STG2DRYMASS+FAIR_MASS+SRB_DRYMASS);
	SetPMI(_V(60,60,1.9));
	SetCrossSections (_V(90,90,4));
	SetCW(0.5,1,1,1);
	SetRotDrag (_V(0.5,0.5,0.1));
	SetTouchdownPoints ( _V(0,-5, -30), _V(-5, 3, -30), _V( 5, 3,-30));

	// camera parameters
	SetCameraOffset (_V(0,2,-5));
	SetCameraRotationRange(PI,PI,(0.4*PI),(0.4*PI));

	oapiReadItem_bool (cfg, "LargeFairing", large_fair);

	SetMeshVisibilityMode(AddMesh("Delta2/delta2stg2"),MESHVIS_ALWAYS);
	if(large_fair == true)
	{
	SetMeshVisibilityMode(AddMesh("Delta2/delta2fair10l_1", &FAIR_OFF2),MESHVIS_ALWAYS);
	SetMeshVisibilityMode(AddMesh("Delta2/delta2fair10l_2", &FAIR_OFF2),MESHVIS_ALWAYS);
	FAIR_MASS = 980;
	}
	else
	{
	SetMeshVisibilityMode(AddMesh("Delta2/delta2fair9_1", &FAIR_OFF),MESHVIS_ALWAYS);
	SetMeshVisibilityMode(AddMesh("Delta2/delta2fair9_2", &FAIR_OFF),MESHVIS_ALWAYS);
	}
	SetMeshVisibilityMode(AddMesh("Delta2/delta2core", &STG1_OFF),MESHVIS_ALWAYS);

	for (int j = 0; j < 3; j++)
	{
	VECTOR3 srb_off;
	MATRIX3 SrbRotMat1 = rotm(_V(0,0,1), (40*RAD*j));
	MATRIX3 SrbRotMat2 = rotm(_V(0,0,1), ((120*RAD)+(40*RAD*j)));
	MATRIX3 SrbRotMat3 = rotm(_V(0,0,1), ((240*RAD)+(40*RAD*j)));
	srb_off = mul(SrbRotMat1,SRB1_OFF);
	SetMeshVisibilityMode(AddMesh("Delta2/gem", &srb_off),MESHVIS_ALWAYS);
	srb_off = mul(SrbRotMat2,SRB1_OFF);
	SetMeshVisibilityMode(AddMesh("Delta2/gem", &srb_off),MESHVIS_ALWAYS);
	srb_off = mul(SrbRotMat3,SRB1_OFF);
	SetMeshVisibilityMode(AddMesh("Delta2/gem", &srb_off),MESHVIS_ALWAYS);
	}

	SetMeshVisibilityMode(AddMesh("Delta2/star_37fm", &STG3_OFF),MESHVIS_ALWAYS);
	SetMeshVisibilityMode(AddMesh("Delta2/yoyos_stowed", &YOYOS_OFF),MESHVIS_ALWAYS);

	if(oapiReadItem_string(cfg, "Adaptor", adname) == TRUE)
	{
	hAdaptor = oapiLoadMeshGlobal(adname);
	SetMeshVisibilityMode(AddMesh(hAdaptor, &ADAPTOR_OFF),MESHVIS_ALWAYS);
	if(hAdaptor) has_adaptor = true;
	}

	DefineAnimations();

	// propellant resources
	
	hProp1 = CreatePropellantResource (STG1FUELMASS,STG1FUELMASS);
	hProp2 = CreatePropellantResource (STG2FUELMASS,STG2FUELMASS);
	hRCSProp2 = CreatePropellantResource (STG2RCSFUELMASS,STG2RCSFUELMASS);
	hProp3 = CreatePropellantResource (STG3FUELMASS,STG3FUELMASS);
	hPropSRB0 = CreatePropellantResource (SRB_FUELMASS,SRB_FUELMASS);
	hPropSRB1 = CreatePropellantResource (SRB_FUELMASS,SRB_FUELMASS);
	hPropSRB2 = CreatePropellantResource (SRB_FUELMASS,SRB_FUELMASS);
	// ***************** thruster definitions *******************

	th_main[2] = CreateThruster (_V( 0, 0, -1.0), _V( 0,0,1), STG3MAINTH0, hProp3, STG3ISP0, STG3ISP0);
	th_main[1] = CreateThruster (_V( 0, 0, -3), _V( 0,0,1), STG2MAINTH1, hProp2, STG2ISPV, STG2ISP0);
	th_main[0] = CreateThruster (_V( 0, 0, -20), _V( 0,0,1), STG1MAINTH1, hProp1, STG1ISPV, STG1ISP0);
	
	
	// srbs

	th_srb[0] = CreateThruster (_V( 0, 0, -20), _V( 0,0,1), SRB_THRUSTV*1.015, hPropSRB0, (SRB_ISPV), (SRB_ISP0));
	th_srb[1] = CreateThruster (_V( 0, 0, -20), _V( 0,0,1), SRB_THRUSTV, hPropSRB1,(SRB_ISPV), (SRB_ISP0));
	th_srb[2] = CreateThruster (_V( 0, 0, -20), _V( 0,0,1), SRB_THRUSTV*1.015, hPropSRB2, (SRB_ISPV), (SRB_ISP0));

	// stg1
	th_rcs[0] = CreateThruster (_V( 0, 2, -20), _V( 1,0,0), STG1RCSTH, hProp1, STG1ISPV, STG1ISP0);
	th_rcs[1] = CreateThruster (_V( 0, 2, -20), _V(-1,0,0), STG1RCSTH, hProp1, STG1ISPV, STG1ISP0);
	th_rcs[2] = CreateThruster (_V( 0,-2, -20), _V( 1,0,0), STG1RCSTH, hProp1, STG1ISPV, STG1ISP0);
	th_rcs[3] = CreateThruster (_V( 0,-2, -20), _V(-1,0,0), STG1RCSTH, hProp1, STG1ISPV, STG1ISP0);

	th_rcs[4] = CreateThruster (_V( 0, 4, -20), _V( 1,0,0), STG1RCSTH, hProp1, STG1ISPV, STG1ISP0);
	th_rcs[5] = CreateThruster (_V( 0, 4, -20), _V(-1,0,0), STG1RCSTH, hProp1, STG1ISPV, STG1ISP0);
	th_rcs[6] = CreateThruster (_V( 0,-4, -20), _V( 1,0,0), STG1RCSTH, hProp1, STG1ISPV, STG1ISP0);
	th_rcs[7] = CreateThruster (_V( 0,-4, -20), _V(-1,0,0), STG1RCSTH, hProp1, STG1ISPV, STG1ISP0);

	th_rcs[8] = CreateThruster (_V( 2, 0, -20), _V(0, 1,0), STG1RCSTH, hProp1, STG1ISPV, STG1ISP0);
	th_rcs[9] = CreateThruster (_V( 2, 0, -20), _V(0,-1,0), STG1RCSTH, hProp1, STG1ISPV, STG1ISP0);
	th_rcs[10] = CreateThruster (_V(-2,0, -20), _V(0, 1,0), STG1RCSTH, hProp1, STG1ISPV, STG1ISP0);
	th_rcs[11] = CreateThruster (_V(-2,0, -20), _V(0,-1,0), STG1RCSTH, hProp1, STG1ISPV, STG1ISP0);


	////// stg2

	th_rcs[12] = CreateThruster (_V( 0, 0, -3), _V( 1,0,0), STG2RCSTH2, hProp2, STG2ISPV, STG2ISP0); //yl
	th_rcs[13] = CreateThruster (_V( 0, 0, -3), _V(-1,0,0), STG2RCSTH2, hProp2, STG2ISPV, STG2ISP0); //  yr
	th_rcs[14] = CreateThruster (_V( 0, 0, -3), _V( 0,-1,0), STG2RCSTH2, hProp2, STG2ISPV, STG2ISP0); // pup
	th_rcs[15] = CreateThruster (_V( 0, 0, -3), _V( 0,1,0), STG2RCSTH2, hProp2, STG2ISPV, STG2ISP0); // pd

	th_rcs[16] = CreateThruster (_V(0.077,0.73,-0.398), _V(-1, 0, 0), STG2RCSTH1, hRCSProp2, STG2RCSISP, STG2RCSISP); // bl/yr
	th_rcs[17] = CreateThruster (_V(-0.077,-0.73,-0.398), _V( 1, 0, 0), STG2RCSTH1, hRCSProp2, STG2RCSISP, STG2RCSISP);// bl/yl
	th_rcs[18] = CreateThruster (_V(0.077,-0.73,-0.398), _V(-1, 0, 0), STG2RCSTH1, hRCSProp2, STG2RCSISP, STG2RCSISP); // br/yr
	th_rcs[19] = CreateThruster (_V(-0.077,0.73,-0.398), _V( 1, 0, 0), STG2RCSTH1, hRCSProp2, STG2RCSISP, STG2RCSISP);// br/yl

	th_rcs[20] = CreateThruster (_V(0,0.79,-0.398), _V( 0,-1, 0), STG2RCSTH1, hRCSProp2, STG2RCSISP, STG2RCSISP); // pu
	th_rcs[21] = CreateThruster (_V(0,0.79,-0.441), _V( 0,-1, 0), STG2RCSTH1, hRCSProp2, STG2RCSISP, STG2RCSISP);// pu
	th_rcs[22] = CreateThruster (_V(0,-0.79,-0.398), _V( 0, 1, 0), STG2RCSTH1, hRCSProp2, STG2RCSISP, STG2RCSISP); // pd
	th_rcs[23] = CreateThruster (_V(0,-0.79,-0.441), _V( 0, 1, 0), STG2RCSTH1, hRCSProp2, STG2RCSISP, STG2RCSISP);// pd

	//// spin table

	th_spin[0] = CreateThruster (_V( -0.663,-0.18,2.928), _V(0,1,0), 0, hRCSProp2, 100000);
	th_spin[1] = CreateThruster (_V( -0.663,-0.18,3.006), _V(0,1,0), 0, hRCSProp2, 100000);

	th_spin[2] = CreateThruster (_V( 0.663,0.18,2.928), _V(0,-1,0), 0, hRCSProp2, 100000);
	th_spin[3] = CreateThruster (_V( 0.663,0.18,3.006), _V(0,-1,0), 0, hRCSProp2, 100000);

	th_spin[4] = CreateThruster (_V( -0.18,0.663,2.928), _V(1,0,0), 0, hRCSProp2, 100000);
	th_spin[5] = CreateThruster (_V( -0.18,0.663,3.006), _V(1,0,0), 0, hRCSProp2, 100000);

	th_spin[6] = CreateThruster (_V( 0.18,-0.663,2.928), _V(-1,0,0), 0, hRCSProp2, 100000);
	th_spin[7] = CreateThruster (_V( 0.18,-0.663,3.006), _V(-1,0,0), 0, hRCSProp2, 100000);


	th_group[0] = th_rcs[9];
	th_group[1] = th_rcs[11];
	htgrppup2 = CreateThrusterGroup (th_group, 2, THGROUP_ATT_PITCHUP);

	th_group[0] = th_rcs[8];
	th_group[1] = th_rcs[10];
	htgrppdown2 = CreateThrusterGroup (th_group, 2, THGROUP_ATT_PITCHDOWN);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[3];
	htgrpyright2 = CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWRIGHT);

	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[2];
	htgrpyleft2 = CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWLEFT);

	th_group[0] = th_rcs[4];
	th_group[1] = th_rcs[7];
	htgrpbright2 = CreateThrusterGroup (th_group, 2, THGROUP_ATT_BANKRIGHT);

	th_group[0] = th_rcs[5];
	th_group[1] = th_rcs[6];
	htgrpbleft2 = CreateThrusterGroup (th_group, 2, THGROUP_ATT_BANKLEFT);


	

	th_group[0] = th_main[0];
	htgrpmain = CreateThrusterGroup (th_group, 1, THGROUP_MAIN);

	SURFHANDLE exhaust = oapiRegisterExhaustTexture ("Exhaust2");
	SURFHANDLE exhaustrcs = oapiRegisterExhaustTexture ("exhaust_atrcs");


	double exlen = 30;
	double exwid = 3;

	AddExhaust(th_main[0],exlen,exwid,_V(0,0,-30.5),_V(0,0,-1),exhaust);

	AddExhaust(th_main[1],15,1.25,_V(0,0,-3.3),_V(0,0,-1),exhaust);

	AddExhaust(th_main[2],10,0.7,_V(0,0,-1.2),_V(0,0,-1),exhaust);

	double exlenrcs = 1;
	double exwidrcs = 0.1;

	AddExhaust(th_rcs[16],exlenrcs,exwidrcs,exhaustrcs);
	AddExhaust(th_rcs[17],exlenrcs,exwidrcs,exhaustrcs);
	AddExhaust(th_rcs[18],exlenrcs,exwidrcs,exhaustrcs);
	AddExhaust(th_rcs[19],exlenrcs,exwidrcs,exhaustrcs);

	AddExhaust(th_rcs[20],exlenrcs,exwidrcs,exhaustrcs);
	AddExhaust(th_rcs[21],exlenrcs,exwidrcs,exhaustrcs);
	AddExhaust(th_rcs[22],exlenrcs,exwidrcs,exhaustrcs);
	AddExhaust(th_rcs[23],exlenrcs,exwidrcs,exhaustrcs);

	/// spin table

	for(int i=0;i<8;i++)
	{
    AddExhaust(th_spin[i], 0.5, 0.04, (exhaust));
	GetThrusterRef(th_spin[i], posth[i]);
	GetThrusterDir(th_spin[i], dirth[i]);
	}

	////// p streams
	SURFHANDLE exhaust2 = oapiRegisterParticleTexture ("Contrail3");

	PARTICLESTREAMSPEC exhaust_main = {
		0, 4, 15, 120, 0.05, 0.15, 0.8, 0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 1,
		exhaust2
	};

	SURFHANDLE smoke1 = oapiRegisterParticleTexture ("Contrail1");

	PARTICLESTREAMSPEC exhaust_srb1 = {
		0, 1.1, 3.5, 125, 0.1, 2, 7, 1.1, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 1,
		smoke1
	};

	PARTICLESTREAMSPEC exhaust_srb2 = {
		0, 3, 4, 160, 0.02, 0.3, 10, 0.4, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 1,
		exhaust2
	};

	if (gcEnabled() == true)
	{
		SURFHANDLE smokeD3D9 = oapiRegisterParticleTexture ("Delta2/delta2_srb_smoke");
		exhaust_srb1.tex = smokeD3D9;
		exhaust_srb1.srcsize = 2;
		exhaust_srb1.srcrate = 5;
		exhaust_srb1.lifetime = 3;
		exhaust_main.srcsize = 5;
		exhaust_main.srcrate = 20;
		exhaust_srb2.lifetime = 0.3;
	}

	///max q pstream
	
	PARTICLESTREAMSPEC maxqtrail = {
		0, 2.0, 20, 50, 0.03, 0.1, 1, 3, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-7, 1,
		smoke1
	};
	AddParticleStream(&maxqtrail,_V(0,0,3.5),_V(0,0,-1), &pst_lvl);
	//////////

	AddExhaustStream(th_main[0],_V( 0, 0,-30.8),&exhaust_main);

	for (int j = 0; j < 3; j++)
	{
	VECTOR3 srb1_ex_off = _V( 1.7, 0,-29.5);
	VECTOR3 srb_ex_off;
	MATRIX3 SrbRotMat1 = rotm(_V(0,0,1), (40*RAD*j));
	MATRIX3 SrbRotMat2 = rotm(_V(0,0,1), ((120*RAD)+(40*RAD*j)));
	MATRIX3 SrbRotMat3 = rotm(_V(0,0,1), ((240*RAD)+(40*RAD*j)));
	srb_ex_off = mul(SrbRotMat1,srb1_ex_off);
	AddExhaustStream(th_srb[j],srb_ex_off,&exhaust_srb1);
	AddExhaustStream(th_srb[j],srb_ex_off,&exhaust_srb2);
	srb_ex_off = mul(SrbRotMat2,srb1_ex_off);
	AddExhaustStream(th_srb[j],srb_ex_off,&exhaust_srb1);
	AddExhaustStream(th_srb[j],srb_ex_off,&exhaust_srb2);
	srb_ex_off = mul(SrbRotMat3,srb1_ex_off);
	AddExhaustStream(th_srb[j],srb_ex_off,&exhaust_srb1);
	AddExhaustStream(th_srb[j],srb_ex_off,&exhaust_srb2);
	}
	///////////////////////////////////////////////////////
	/////AirFoils

	CreateAirfoil (LIFT_VERTICAL, _V(0,0,0), VLiftCoeff, 10, 25, 0.16);
	CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,0), HLiftCoeff, 10, 25, 0.16);

	////////

	pld_attach[0] = CreateAttachment (false, PLD_ATT, _V( 0,0,1), _V(0,1,0), "PLD");
	pld_attach[1] = CreateAttachment (false, PLD_ATT, _V( 0,0,1), _V(0,1,0), "PLD");
	pld_attach[2] = CreateAttachment (false, PLD_ATT, _V( 0,0,1), _V(0,1,0), "PLD");
	lpd_attach = CreateAttachment (true, _V( 0, 0, -28.8), _V( 0,0,-1), _V(0,-1,0), "LP");

	//local light
	COLOUR4 col_d = {0.125,0.1,0.06125,0};
	COLOUR4 col_s = {0,0,0,0};
	COLOUR4 col_a = {0,0,0,0};
	eng_light = (PointLight*)AddPointLight(_V(0,0,-31),5000, 1e-1, 0, 1e-6, col_d, col_s, col_a);
	eng_light->Activate(false);

}

void delta7926::clbkPostCreation ()
{
 hGref = GetGravityRef();

 if(yoyo_status == 3)DelMesh(14);

if (srb_status == 1)
{
	DelMesh(12);
	DelMesh(11);
	DelMesh(10);
}
else if (srb_status == 2)
{
	DelMesh(12);
	DelMesh(11);
	DelMesh(10);
	DelMesh(9);
	DelMesh(8);
	DelMesh(7);
}
else if (srb_status == 3)
{
	DelMesh(12);
	DelMesh(11);
	DelMesh(10);
	DelMesh(9);
	DelMesh(8);
	DelMesh(7);
	DelMesh(6);
	DelMesh(5);
	DelMesh(4);
}

if(fair_status == 1)
{
	DelMesh(2);
	DelMesh(1);
}

if (eng_status == 1)
{
	 DelMesh(3);
	 ConfigMain();
}
else if (eng_status == 2)
{
	 DelMesh(3);
	 DelMesh(0);
	 ConfigMain2();
}
CalcEmptyMass();

	ap_tgt = 0.95*op_meco_alt;
	if (ap_tgt < 100000)ap_tgt = 100000;
 
	double lat, lng, rds;
	GetEquPos(lng,lat,rds);
	if (abs(tgt_inc) < abs(lat)) tgt_inc = lat;

	///////////// sound
	MyID=ConnectToOrbiterSoundDLL(GetHandle());

	RequestLoadVesselWave(MyID,CTDN,"Sound\\Vessel\\countdown1.wav",DEFAULT);
	RequestLoadVesselWave(MyID,SEP,"Sound\\Vessel\\undocking.wav",BOTHVIEW_FADED_FAR);
	
	
	SoundOptionOnOff(MyID,PLAYRADIOATC,FALSE);
	SoundOptionOnOff(MyID,PLAYCOUNTDOWNWHENTAKEOFF,FALSE);
	SoundOptionOnOff(MyID,PLAYLANDINGANDGROUNDSOUND,FALSE);
	SoundOptionOnOff(MyID,PLAYCABINAIRCONDITIONING,FALSE);
	SoundOptionOnOff(MyID,PLAYCABINRANDOMAMBIANCE,FALSE);
	SoundOptionOnOff(MyID,DISPLAYTIMER,FALSE);
	SoundOptionOnOff(MyID,PLAYDOCKLANDCLEARANCE,FALSE);
}


// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new delta7926 (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (delta7926*)vessel;
}


void delta7926::clbkSaveState (FILEHANDLE scn)
{
	char cbuf[256];


	// default vessel parameters
	VESSEL3::clbkSaveState(scn);

	//sprintf (cbuf, "%d %0.4f",);
	//oapiWriteScenario_string (scn, "LEGS", cbuf);

	sprintf (cbuf, "%d",fair_status);
	oapiWriteScenario_string (scn, "FAIR", cbuf);

	sprintf (cbuf, "%d",srb_status);
	oapiWriteScenario_string (scn, "SRBS", cbuf);

	sprintf (cbuf, "%d %0.4f %d %0.4f %0.4f",spin_status, spin_proc, yoyo_status, yoyo_proc, drot);
	oapiWriteScenario_string (scn, "ANMS", cbuf);

	sprintf (cbuf, "%d %d %d %d %d", eng_status, ap_status, ap_function, op_status, start_sw);
	oapiWriteScenario_string (scn, "SOST", cbuf);

	sprintf (cbuf, "%0.4f %0.1f %0.1f %0.1f %0.2f %0.4f", tgt_inc, op_meco_alt, op_tgt, ap_tgt, timer, pitch_angle);
	oapiWriteScenario_string (scn, "SOPR", cbuf);


}

void delta7926::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
    char *line;
	while (oapiReadScenario_nextline (scn, line)) {
		if (!strnicmp (line, "FAIR", 4)) {
		sscanf (line+4, "%d", &fair_status);
		} else if (!strnicmp (line, "SRBS", 4)) {
		sscanf (line+4, "%d", &srb_status);
		} else if (!strnicmp (line, "ANMS", 4)) {
		sscanf (line+4, "%d%lf%d%lf%lf", &spin_status, &spin_proc, &yoyo_status, &yoyo_proc, &drot);
		} else if (!strnicmp (line, "SOST", 4)) {
		sscanf (line+4, "%d%d%d%d%d", &eng_status, &ap_status, &ap_function, &op_status, &start_sw);
		} else if (!strnicmp (line, "SOPR", 4)) {
			sscanf (line+4, "%lf%lf%lf%lf%lf%lf",&tgt_inc, &op_meco_alt, &op_tgt, &ap_tgt, &timer, &pitch_angle);
		}else {
            ParseScenarioLineEx (line, vs);
			// unrecognised option - pass to Orbiter's generic parser
        }
   }

	

}

bool SetINC(void *id, char *str, void *data)
{
	float new_inc;
	sscanf(str, "%f", &new_inc);
	double lat, lng, rds;
	oapiGetEquPos(oapiGetFocusObject(),&lng,&lat,&rds);
	if (abs(new_inc) < abs(lat*DEG))return false;
	else 
		{
		char abuf[256];
		sprintf (abuf, "%0.4f", new_inc);
		VESSEL2* vme = (VESSEL2*)oapiGetVesselInterface(oapiGetFocusObject ());
		vme->SendBufferedKey(COMM_SET_INC, false, abuf);
		return true;
		}
}


bool SetOPE(void *id, char *str, void *data)
{
	float new_ope;
	sscanf(str, "%f", &new_ope);
	if (new_ope < 160)return false;
	else 
		{
		char abuf[256];
		sprintf (abuf, "%0.4f", new_ope);
		VESSEL2* vme = (VESSEL2*)oapiGetVesselInterface(oapiGetFocusObject ());
		vme->SendBufferedKey(COMM_SET_PER, false, abuf);
		return true;
		}
}

bool SetOAP(void *id, char *str, void *data)
{
	float new_oap;
	sscanf(str, "%f", &new_oap);
	if (new_oap < 160)return false;
	else 
		{
		char abuf[256];
		sprintf (abuf, "%0.4f", new_oap);
		VESSEL2* vme = (VESSEL2*)oapiGetVesselInterface(oapiGetFocusObject ());
		vme->SendBufferedKey(COMM_SET_APO, false, abuf);	
		return true;
		}
}

bool SetPLD(void *id, char *str, void *data)
{
	char cbuf[256];
	sscanf(str, "%s", &cbuf);
	OBJHANDLE hPld = oapiGetObjectByName(cbuf);
	if (hPld == NULL)return false;
	else
	{
	VESSEL2* vme = (VESSEL2*)oapiGetVesselInterface(oapiGetFocusObject ());
	if(vme->SendBufferedKey(COMM_SET_PLD, false, cbuf) == 1)return true;	
	else return false;
	}
}

int delta7926::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate) {

	if (key == COMM_SET_INC) 
	{
		sscanf (kstate, "%lf", &tgt_inc);
		tgt_inc = tgt_inc*RAD;
		return 1;
	}

	if (key == COMM_SET_PER) 
	{
		double new_ope;
		sscanf (kstate, "%lf", &new_ope);
		if (new_ope > op_tgt/1000)
			{
			op_meco_alt = op_tgt;
			op_tgt = new_ope*1000;
			} 
			else op_meco_alt = new_ope*1000;
			ap_tgt = 0.95*op_meco_alt;
			if (ap_tgt < 100000)ap_tgt = 100000;
		return 1;
	}

	if (key == COMM_SET_APO) 
	{
		double new_oap;
		sscanf (kstate, "%lf", &new_oap);
		if (new_oap < op_meco_alt/1000)
			{
			op_tgt = op_meco_alt;
			op_meco_alt = new_oap*1000;
			} 
			else op_tgt = new_oap*1000;
			ap_tgt = 0.95*op_meco_alt;
			if (ap_tgt < 100000)ap_tgt = 100000;
		return 1;
	}

	if (key == COMM_SET_PLD) 
	{
	int att_index = 0;
	if(GetAttachmentStatus(pld_attach[0]) == NULL)att_index = 0;
	else if(GetAttachmentStatus(pld_attach[1]) == NULL)att_index = 1;
	else if(GetAttachmentStatus(pld_attach[2]) == NULL)att_index = 2;
	else return 0;

	char new_pld_name[256];
	sscanf (kstate, "%s", &new_pld_name);
	OBJHANDLE hPld = oapiGetObjectByName(new_pld_name);

	VESSEL2* v = (VESSEL2*)oapiGetVesselInterface(hPld);
	DWORD ac = v->AttachmentCount(true);
	if (ac == 0) return 0;
	for (DWORD j = 0; j < ac; j++)
		{
		ATTACHMENTHANDLE hAtt = v->GetAttachmentHandle(true, j);
		OBJHANDLE hOp = v->GetAttachmentStatus(hAtt);
		if (hOp == NULL)
			{
			if (AttachChild(hPld, pld_attach[att_index], hAtt) == true)
				{
					pld_att_status = pld_att_status + 1;
					calc_sw = 0;
					return 1;
				}
			else return 0;
			}
		else if (j == ac-1) return 0;
		}
		return 1;
	}

if (!down) return 0; // only process keydown events

  if (KEYMOD_SHIFT (kstate)) 
  {	 
	switch (key)
	{
	

	} 
  }

  else if (KEYMOD_CONTROL (kstate)) 
  { 
	switch (key)
	{
		
	} 
  }

else { // unmodified keys
    switch (key) {

		case OAPI_KEY_SUBTRACT: // disable retros
		RESETKEY (kstate, OAPI_KEY_SUBTRACT);
		return 1;

		case OAPI_KEY_G: // spinup and jettison stg2
		if(fair_status == 1 && eng_status == 1)
		{
		if(spin_status == 0)spin_status = 1;
		}
		return 1;

		case OAPI_KEY_K:
		oapiOpenInputBox("Enter Apogee Alt.> 160 (km)",SetOAP, 0, 50, 0);
		return 1;

		case OAPI_KEY_M:
		oapiOpenInputBox("Enter Perigee Alt.> 160 (km)",SetOPE, 0, 50, 0);
		return 1;

		case OAPI_KEY_L: // enter inc
		RESETKEY (kstate, OAPI_KEY_L);
		//double lat, lng, rds;
		//GetEquPos(lng,lat,rds);
		//min_inc = DEG*lat;
		oapiOpenInputBox("Enter Equ.Inc(+North : -South)",SetINC, 0, 50, 0);
		return 1;
		
		case OAPI_KEY_P: // add payload
		if(pld_att_status < 3)oapiOpenInputBox("Enter Payload Name", SetPLD, 0, 50, 0);
		return 1;

		case OAPI_KEY_J: // jettison pld
		if (fair_status == 1)
		{
		
			if(DetachChild(pld_attach[2], 0.3) == true)pld_att_status = pld_att_status - 1;
			else if(DetachChild(pld_attach[1], 0.3) == true)pld_att_status = pld_att_status - 1;
			else if(DetachChild(pld_attach[0], 0.3) == true)pld_att_status = pld_att_status - 1;
			CalcEmptyMass();
			
		}
		else if (fair_status == 0)
		{
		JettisonFair();
		}
		return 1;

		

		

		case OAPI_KEY_V: // ascent pilot
		if (eng_status == 0)
			{
			if(ap_status == 0)ap_status = 1;
			else ap_status = 0;
			op_status = 0;
			KillAllRCS2();
			}
		else if (eng_status == 1)
			{
			if(op_status == 0)op_status = 1;
			else op_status = 0;
			ap_status = 0;
			KillAllRCS2();
			}
		return 1;

		case OAPI_KEY_E:
			if (cam_status == 0)cam_status = 1;
			else if (cam_status == 1)cam_status = 0;
			SetCamDir();
		return 1;

		//case OAPI_KEY_B:
		//return 1;

		case OAPI_KEY_N:
		if (eng_status == 0)JettisonStg();
		return 1;
	}
  }
  return 0;
}




void delta7926::clbkPreStep(double SimT, double SimDT, double MJD)
{

double dynp = GetDynPressure();
if(dynp > 30000)
	{
	pst_lvl = (dynp-30000)/3000;
	if (pst_lvl > 1)pst_lvl = 1;
	if(eng_status > 0)pst_lvl = 0;
	}
else pst_lvl = 0;

step_time = SimDT;///////
double simdt = SimDT;



mth_lev = GetThrusterLevel(th_main[eng_status]);
if(eng_status == 1)
{
if (mth_lev == 0 && rcs_sw2 == 0) ConfigRCS2();
else if (mth_lev > 0 && rcs_sw2 == 1) RevertRCS2();
}

else if (srb_status == 0)
{
	if(GetPropellantMass(hPropSRB2) > 0)
	{
		if(mth_lev > 0.95)SetThrusterLevel(th_srb[2],1);
	}
	else
	{
		srb_status = 1;
		SetThrusterLevel(th_srb[2],0);
		JettisonSRB2();
	}

}

if (srb_status < 2)
{
	if(GetPropellantMass(hPropSRB1) > 0)
	{
		if(mth_lev > 0.95)SetThrusterLevel(th_srb[1],1);
	}
	else
	{
		srb_status = 2;
		SetThrusterLevel(th_srb[1],0);
		JettisonSRB1();
	}

}
else if (srb_status == 2)
{
if(GetPropellantMass(hPropSRB0) > 0)SetThrusterLevel(th_srb[0],1);
else
	{
		srb_status = 3;
		SetThrusterLevel(th_srb[0],0);
		JettisonSRB0();
	}
}

if (calc_sw == 0)
{
CalcEmptyMass();
calc_sw = 1;
}

if (ap_status == 1) DoAscent();
else if (op_status == 1) DoOrbit();

////////////////engine light
if (mth_lev > 0)
	{
	double rndm = 0.15*(oapiRand());
	rndm = (mth_lev*0.85) - rndm;
	if (rndm < 0)rndm = 0;
	else if (rndm > 1)rndm = 1;
	eng_light->Activate(true);
	eng_light->SetIntensity(rndm);
	}
	else eng_light->Activate(false);
///////////////////////////////

////////// animations ////////////////////////////

if (spin_status == 1)
{
	drot = drot + (0.25*0.25*step_time);
	if(drot > 0.25)
	{
		drot = 0.25;
		spin_status = 2;
	}
	spin_proc = spin_proc + (drot*step_time);
	if(spin_proc > 1) spin_proc = spin_proc - ((int)(spin_proc));

	for (int i = 0; i < 8; i++)
	{
	SetThrusterLevel(th_spin[i], 1);
	}
	SetSpinParams();
	SetAnimation(anim_spin, spin_proc);
}
else if (spin_status == 2)
{
	drot = 0.25;
	if((1 - spin_proc) < (drot*step_time))
	{
	spin_status = 3;
	spin_proc = 0;
	SetSpinParams();
	SetAnimation(anim_spin, spin_proc);
	JettisonStg2();
	}
	else
	{
	spin_proc = spin_proc + (drot*step_time);
	if(spin_proc > 1) spin_proc = spin_proc - ((int)(spin_proc));
	SetSpinParams();
	SetAnimation(anim_spin, spin_proc);
	}


	for (int i = 0; i < 8; i++)
	{
	SetThrusterLevel(th_spin[i], 0);
	}
}

////////// yoyos
if (eng_status == 2 && GetThrusterLevel(th_main[2]) > 0) yoyo_status = 1;
if (yoyo_status == 1)
	{
	SetThrusterLevel(th_main[2], 1);
	if (GetPropellantMass(hProp3) == 0)
		{
		SetThrusterLevel(th_main[2], 0);
		yoyo_status = 2;
		}
	}
if (yoyo_status == 2)
	{
		yoyo_proc = yoyo_proc + (step_time*0.25);
		if (yoyo_proc > 1)
		{
			yoyo_proc = 1;
			SetAnimation(anim_yoyo, yoyo_proc);
			JettisonYoyos();
			yoyo_status = 3;
		}
		else
		{
		VECTOR3 ang_vel;
		GetAngularVel(ang_vel);
		ang_vel.z = ang_vel.z - (0.22*step_time*PI05);
		SetAngularVel(ang_vel);
		SetAnimation(anim_yoyo, yoyo_proc);
		}
		
	}


}

void delta7926::SetSpinParams(void)
{
	VECTOR3 rot_axis = _V(0,0,1);
	double rot_angle = PI2*spin_proc;
	VECTOR3 new_att_rot = _V((sin(rot_angle)),(cos(rot_angle)),0);
	for (int i = 0; i < 3; i++)
	{
	SetAttachmentParams(pld_attach[i], PLD_ATT, _V(0,0,1), new_att_rot);
	}

	if(spin_status == 1)
	{
	MATRIX3 rotmat = rotm(rot_axis, -rot_angle);
	for (int k = 0; k < 8; k++)
		{
		VECTOR3 new_pos = mul(rotmat, posth[k]);
		VECTOR3 new_dir = mul(rotmat, dirth[k]);
		SetThrusterRef(th_spin[k], new_pos);
		SetThrusterDir(th_spin[k], new_dir);
		}
	}

}

void delta7926::clbkPostStep (double simt, double simdt, double mjd)
{
	
}




bool delta7926::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp)
{
	// draw the default HUD
	VESSEL3::clbkDrawHUD (mode, hps, skp);

	int s = hps->H;
	double d = (s*0.00130208);
	int sw = ((hps->W));
	int lw =  (int)(16*sw/1024);
	int lwoffset = sw - (18*lw);
	int hlw = (int) (lw/2);

	//int chw = 16;

	
	char test[256];
	strcpy(test,"abcdefghijklmnopqrstuvwxyz");
	int it = skp->GetTextWidth(test, 26);
	it = (int)(it/26);
	int chw = it;

	int roxl = 0;
	int royl = 0;

	double ds = s;
	double dsw = sw;
	double sc_ratio = ds/dsw;


	if (sc_ratio < 0.7284)
	{
	roxl = (lw*10);
	royl = (int) (-88*d);
	}



	int wd = (int)(152*d);
	int wc = (int)(168*d);
	int w0 = (int)(184*d);
	int w1 = (int) (200*d);
	int w2 = (int)(216*d);
	int w3 = (int)(232*d);
	int w4 = (int)(248*d);
	int w5 = (int)(264*d);
	int w6 = (int)(280*d);
	int w7 = (int)(296*d);
	int w8 = (int)(312*d);
	int w9 = (int)(328*d);
	int w10 = (int)(344*d);
	//int w11 = (int)(360*d);
	//int w12 = (int)(376*d);
	//int w13 = (int)(392*d);
	//int w14 = (int)(408*d);
	//int w15 = (int)(424*d);


	if (pld_att_status < 3)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"P = Attach Payload");
	skp->Text ((10+roxl), (wd+royl), abuf,strlen(abuf));
	}
	
	{
	skp->SetTextColor(0x00FFFFFF);
	char abuf[256];
	sprintf(abuf,"Payload_1: %s %dkg", pldname1, int(pld_mass1));
	skp->Text ((10+roxl), (wc+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x00FFFFFF);
	char abuf[256];
	sprintf(abuf,"Payload_2: %s %dkg", pldname2, int(pld_mass2));
	skp->Text ((10+roxl), (w0+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x00FFFFFF);
	char abuf[256];
	sprintf(abuf,"Payload_3: %s %dkg", pldname3, int(pld_mass3));
	skp->Text ((10+roxl), (w1+royl), abuf,strlen(abuf));
	}


	if (eng_status == 1)
	{
	skp->SetTextColor(0x00FFFFFF);
	char abuf[256];
	double m1 = GetMass();
	double m2 = 0;
	if (eng_status == 0)m2 = GetPropellantMass(hProp1);
	else if (eng_status == 1)m2 = GetPropellantMass(hProp2);
	else m2 = GetPropellantMass(hProp3);
	double test = (m1/(m1-m2));
	double isp_curr = GetThrusterIsp(th_main[eng_status]);
	double zdv = (isp_curr*(log(test)));

	sprintf(abuf,"2nd Stage dV %0.2fm/s : RCS Prop. %0.1fkg", zdv, GetPropellantMass(hRCSProp2));
	skp->Text ((10+roxl), (w2+royl), abuf,strlen(abuf));
	}
	
	{
	skp->SetTextColor(0x00FFFFFF);
	char abuf[256];
	double m1 = STG3DRYMASS + GetPropellantMass(hProp3) + pld_mass1 + pld_mass2 + pld_mass3;
	double m2 = STG3DRYMASS + pld_mass1 + pld_mass2 + pld_mass3;

	double test = (m1/m2);
	double isp_curr = GetThrusterIsp(th_main[2]);
	double zdv = (isp_curr*(log(test)));

	sprintf(abuf,"3rd Stage dV %0.2f m/s", zdv);
	skp->Text ((10+roxl), (w3+royl), abuf,strlen(abuf));
	}



	{
	skp->SetTextColor(0x00FFFFFF);
	char abuf[256];
	sprintf(abuf,"L = Enter Orbit Inc: %0.2f",(tgt_inc*DEG));
	skp->Text ((10+roxl), (w4+royl), abuf,strlen(abuf));
	}


	{
	skp->SetTextColor(0x00FFFFFF);
	char abuf[256];
	sprintf(abuf,"K = Enter Orbit Apogee: %dkm",int(op_tgt/1000));
	skp->Text ((10+roxl), (w5+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x00FFFFFF);
	char abuf[256];
	sprintf(abuf,"M = Enter Orbit Perigee: %dkm",int(op_meco_alt/1000));
	skp->Text ((10+roxl), (w6+royl), abuf,strlen(abuf));
	}

	if (ap_status == 0 && op_status == 0)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"V = Start Launch Autopilot");
	skp->Text ((10+roxl), (w7+royl), abuf,strlen(abuf));
	}
	else
	{
	skp->SetTextColor(0x0066FFFF);
	char abuf[256];
	sprintf(abuf,"V = Launch Autopilot Active");
	skp->Text ((10+roxl), (w7+royl), abuf,strlen(abuf));
	}


	if (eng_status == 0)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"N = Jettison 1st Stage");
	skp->Text ((10+roxl), (w8+royl), abuf,strlen(abuf));
	}

	if (eng_status == 1 && fair_status == 1)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"G = Spin-Up/Jettison 3rd Stage");
	skp->Text ((10+roxl), (w8+royl), abuf,strlen(abuf));
	}


	if (fair_status == 0)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"J = Jettison Fairing");
	skp->Text ((10+roxl), (w9+royl), abuf,strlen(abuf));
	}
	else if (pld_att_status > 0)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"J = Jettison Payload");
	skp->Text ((10+roxl), (w9+royl), abuf,strlen(abuf));
	}

	


	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"E = Set View +Z/-Z");
	skp->Text ((10+roxl), (w10+royl), abuf,strlen(abuf));
	}

	return true;
}

void delta7926::RevertRCS2 (void)
{
	SetThrusterGroupLevel(htgrppup2,0);
	SetThrusterGroupLevel(htgrppdown2,0);
	SetThrusterGroupLevel(htgrpyright2,0);
	SetThrusterGroupLevel(htgrpyleft2,0);

	DelThrusterGroup(htgrppup2,THGROUP_ATT_PITCHUP, false);
	DelThrusterGroup(htgrppdown2,THGROUP_ATT_PITCHDOWN, false);
	DelThrusterGroup(htgrpyright2,THGROUP_ATT_YAWRIGHT, false);
	DelThrusterGroup(htgrpyleft2,THGROUP_ATT_YAWLEFT, false);

	th_group[0] = th_rcs[14];
	htgrppup2 = CreateThrusterGroup (th_group, 1, THGROUP_ATT_PITCHUP);

	th_group[0] = th_rcs[15];
	htgrppdown2 = CreateThrusterGroup (th_group, 1, THGROUP_ATT_PITCHDOWN);

	th_group[0] = th_rcs[13];
	htgrpyright2 = CreateThrusterGroup (th_group, 1, THGROUP_ATT_YAWRIGHT);

	th_group[0] = th_rcs[12];
	htgrpyleft2 = CreateThrusterGroup (th_group, 1, THGROUP_ATT_YAWLEFT);

	rcs_sw2 = 0;
}

void delta7926::ConfigRCS2 (void)
{
	SetThrusterGroupLevel(htgrppup2,0);
	SetThrusterGroupLevel(htgrppdown2,0);
	SetThrusterGroupLevel(htgrpyright2,0);
	SetThrusterGroupLevel(htgrpyleft2,0);

	DelThrusterGroup(htgrppup2,THGROUP_ATT_PITCHUP, false);
	DelThrusterGroup(htgrppdown2,THGROUP_ATT_PITCHDOWN, false);
	DelThrusterGroup(htgrpyright2,THGROUP_ATT_YAWRIGHT, false);
	DelThrusterGroup(htgrpyleft2,THGROUP_ATT_YAWLEFT, false);

	th_group[0] = th_rcs[20];
	th_group[1] = th_rcs[21];
	htgrppup2 = CreateThrusterGroup (th_group, 2, THGROUP_ATT_PITCHUP);

	th_group[0] = th_rcs[22];
	th_group[1] = th_rcs[23];
	htgrppdown2 = CreateThrusterGroup (th_group, 2, THGROUP_ATT_PITCHDOWN);

	th_group[0] = th_rcs[16];
	th_group[1] = th_rcs[18];
	htgrpyright2 = CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWRIGHT);

	th_group[0] = th_rcs[17];
	th_group[1] = th_rcs[19];
	htgrpyleft2 = CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWLEFT);

	rcs_sw2 = 1;
}

void delta7926::JettisonSRB0 (void)
{
MATRIX3 SrbRotMat1 = rotm(_V(0,0,1),(120*RAD));
MATRIX3 SrbRotMat2 = rotm(_V(0,0,1),(240*RAD));


VESSELSTATUS vs;
GetStatus(vs);
VECTOR3 rvel;
VECTOR3 srb_vel_off = _V(10,0,0);
VECTOR3 srb_vrot_off = _V(0,-1.2,0);
VECTOR3 srb_pos_off = SRB1_OFF;
Local2Rel (srb_pos_off, vs.rpos);
GlobalRot (srb_vel_off, rvel);
vs.rvel = vs.rvel + rvel;
vs.vrot = vs.vrot + srb_vrot_off;
vs.eng_main = vs.eng_hovr = 0.0;
vs.status = 0;
oapiCreateVessel ("Delta_SRB_9", "Delta2/delta2_srb", vs);

GetStatus(vs);
srb_vel_off = mul(SrbRotMat1, _V(10,0,0));
srb_vrot_off = mul(SrbRotMat1, _V(0,-1.2,0));
srb_pos_off = mul(SrbRotMat1, SRB1_OFF);
Local2Rel (srb_pos_off, vs.rpos);
GlobalRot (srb_vel_off, rvel);
vs.rvel = vs.rvel + rvel;
vs.vrot = vs.vrot + srb_vrot_off;
vs.eng_main = vs.eng_hovr = 0.0;
vs.status = 0;
oapiCreateVessel ("Delta_SRB_8", "Delta2/delta2_srb", vs);

GetStatus(vs);
srb_vel_off = mul(SrbRotMat2, _V(10,0,0));
srb_vrot_off = mul(SrbRotMat2, _V(0,-1.2,0));
srb_pos_off = mul(SrbRotMat2, SRB1_OFF);
Local2Rel (srb_pos_off, vs.rpos);
GlobalRot (srb_vel_off, rvel);
vs.rvel = vs.rvel + rvel;
vs.vrot = vs.vrot + srb_vrot_off;
vs.eng_main = vs.eng_hovr = 0.0;
vs.status = 0;
oapiCreateVessel ("Delta_SRB_7", "Delta2/delta2_srb", vs);

DelMesh(6);
DelMesh(5);
DelMesh(4);

DelPropellantResource(hPropSRB0);
CalcEmptyMass();
}

void delta7926::JettisonSRB1 (void)
{
MATRIX3 SrbRotMat1 = rotm(_V(0,0,1),(40*RAD));
MATRIX3 SrbRotMat2 = rotm(_V(0,0,1),(160*RAD));
MATRIX3 SrbRotMat3 = rotm(_V(0,0,1),(280*RAD));


VESSELSTATUS vs;
GetStatus(vs);
VECTOR3 rvel;
VECTOR3 srb_vel_off = mul(SrbRotMat1, _V(10,0,0));
VECTOR3 srb_vrot_off =mul(SrbRotMat1, _V(0,-1.2,0));
VECTOR3 srb_pos_off = mul(SrbRotMat1, SRB1_OFF);
Local2Rel (srb_pos_off, vs.rpos);
GlobalRot (srb_vel_off, rvel);
vs.rvel = vs.rvel + rvel;
vs.vrot = vs.vrot + srb_vrot_off;
vs.eng_main = vs.eng_hovr = 0.0;
vs.status = 0;
oapiCreateVessel ("Delta_SRB_6", "Delta2/delta2_srb", vs);

GetStatus(vs);
srb_vel_off = mul(SrbRotMat2, _V(10,0,0));
srb_vrot_off = mul(SrbRotMat2, _V(0,-1.2,0));
srb_pos_off = mul(SrbRotMat2, SRB1_OFF);
Local2Rel (srb_pos_off, vs.rpos);
GlobalRot (srb_vel_off, rvel);
vs.rvel = vs.rvel + rvel;
vs.vrot = vs.vrot + srb_vrot_off;
vs.eng_main = vs.eng_hovr = 0.0;
vs.status = 0;
oapiCreateVessel ("Delta_SRB_5", "Delta2/delta2_srb", vs);

GetStatus(vs);
srb_vel_off = mul(SrbRotMat3, _V(10,0,0));
srb_vrot_off = mul(SrbRotMat3, _V(0,-1.2,0));
srb_pos_off = mul(SrbRotMat3, SRB1_OFF);
Local2Rel (srb_pos_off, vs.rpos);
GlobalRot (srb_vel_off, rvel);
vs.rvel = vs.rvel + rvel;
vs.vrot = vs.vrot + srb_vrot_off;
vs.eng_main = vs.eng_hovr = 0.0;
vs.status = 0;
oapiCreateVessel ("Delta_SRB_4", "Delta2/delta2_srb", vs);

DelMesh(9);
DelMesh(8);
DelMesh(7);

DelPropellantResource(hPropSRB1);
CalcEmptyMass();
}


void delta7926::JettisonSRB2 (void)
{
MATRIX3 SrbRotMat1 = rotm(_V(0,0,1),(80*RAD));
MATRIX3 SrbRotMat2 = rotm(_V(0,0,1),(200*RAD));
MATRIX3 SrbRotMat3 = rotm(_V(0,0,1),(320*RAD));


VESSELSTATUS vs;
GetStatus(vs);
VECTOR3 rvel;
VECTOR3 srb_vel_off = mul(SrbRotMat1, _V(10,0,0));
VECTOR3 srb_vrot_off =mul(SrbRotMat1, _V(0,-1.2,0));
VECTOR3 srb_pos_off = mul(SrbRotMat1, SRB1_OFF);
Local2Rel (srb_pos_off, vs.rpos);
GlobalRot (srb_vel_off, rvel);
vs.rvel = vs.rvel + rvel;
vs.vrot = vs.vrot + srb_vrot_off;
vs.eng_main = vs.eng_hovr = 0.0;
vs.status = 0;
oapiCreateVessel ("Delta_SRB_3", "Delta2/delta2_srb", vs);

GetStatus(vs);
srb_vel_off = mul(SrbRotMat2, _V(10,0,0));
srb_vrot_off = mul(SrbRotMat2, _V(0,-1.2,0));
srb_pos_off = mul(SrbRotMat2, SRB1_OFF);
Local2Rel (srb_pos_off, vs.rpos);
GlobalRot (srb_vel_off, rvel);
vs.rvel = vs.rvel + rvel;
vs.vrot = vs.vrot + srb_vrot_off;
vs.eng_main = vs.eng_hovr = 0.0;
vs.status = 0;
oapiCreateVessel ("Delta_SRB_2", "Delta2/delta2_srb", vs);

GetStatus(vs);
srb_vel_off = mul(SrbRotMat3, _V(10,0,0));
srb_vrot_off = mul(SrbRotMat3, _V(0,-1.2,0));
srb_pos_off = mul(SrbRotMat3, SRB1_OFF);
Local2Rel (srb_pos_off, vs.rpos);
GlobalRot (srb_vel_off, rvel);
vs.rvel = vs.rvel + rvel;
vs.vrot = vs.vrot + srb_vrot_off;
vs.eng_main = vs.eng_hovr = 0.0;
vs.status = 0;
oapiCreateVessel ("Delta_SRB_1", "Delta2/delta2_srb", vs);

DelMesh(12);
DelMesh(11);
DelMesh(10);

DelPropellantResource(hPropSRB2);
CalcEmptyMass();
}

void delta7926::JettisonFair (void)
{
VESSELSTATUS vs;
 GetStatus(vs); // get mother vessel status vs
  VECTOR3 rvel;
  Local2Rel (FAIR_OFF, vs.rpos); // child vessel position absolute in global frame
  GlobalRot (_V(4,0,0), rvel);  // child vessel velocity relative to mother, convert to global frame
  vs.rvel = vs.rvel + rvel;  // child vessel velocity absolute in global frame
  vs.eng_main = vs.eng_hovr = 0.0; // engines off
  vs.status = 0; // in flight
  vs.vrot.x = vs.vrot.x;	// child vessel rotation relative to mother
  vs.vrot.z = vs.vrot.z;	// child vessel rotation relative to mother
  vs.vrot.y = vs.vrot.y - 0.2; // child vessel rotation relative to mother
  if(large_fair == true) oapiCreateVessel ("Delta_Fair_1", "Delta2/delta2_fair10l_1", vs);
  else oapiCreateVessel ("Delta_Fair_1", "Delta2/delta2_fair9_1", vs); // ("child name", "cfg path", vesselstatus vs)

  GetStatus(vs);
  Local2Rel (FAIR_OFF, vs.rpos);
  GlobalRot (_V(-4,0,0), rvel);
  vs.rvel = vs.rvel + rvel;
  vs.eng_main = vs.eng_hovr = 0.0;
  vs.status = 0;
  vs.vrot.x = vs.vrot.x;
  vs.vrot.z = vs.vrot.z;
  vs.vrot.y = vs.vrot.y + 0.2;
  if(large_fair == true) oapiCreateVessel ("Delta_Fair_2", "Delta2/delta2_fair10l_2", vs);
  else oapiCreateVessel ("Delta_Fair_2", "Delta2/delta2_fair9_2", vs);

  
  
  DelMesh(2);
  DelMesh(1);
  

fair_status = 1;
CalcEmptyMass();

PlayVesselWave(MyID, SEP, 0, 200, 30000);
}


void delta7926::JettisonStg (void)
{
	if (srb_status == 0)
	{
		JettisonSRB0();
		JettisonSRB1();
		JettisonSRB2();
		srb_status = 3;
	}
	else if (srb_status == 1)
	{
		JettisonSRB0();
		JettisonSRB1();
		srb_status = 3;
	}
	else if (srb_status == 2)
	{
		JettisonSRB0();
		srb_status = 3;
	}

  VESSELSTATUS vs;
  //char name[256];
  GetStatus(vs);
  VECTOR3 rvel;
  Local2Rel (STG1_OFF, vs.rpos);
  GlobalRot (_V(0,0,-5), rvel);
  vs.rvel = vs.rvel + rvel;
  vs.eng_main = vs.eng_hovr = 0.0;
  vs.status = 0;
  vs.vrot.x = vs.vrot.x;
  vs.vrot.z = vs.vrot.z;
  vs.vrot.y = vs.vrot.y;
  OBJHANDLE hStg1 = oapiCreateVessel ("Delta2_Stg1", "Delta2/delta2_core", vs);
  
	DelMesh(3);
	eng_status = 1;


	CalcEmptyMass();
	ConfigMain();

	PlayVesselWave(MyID, SEP, 0, 200, 25000);

}


void delta7926::JettisonStg2 (void)
{

  VESSELSTATUS vs;
  //char name[256];
  GetStatus(vs);
  VECTOR3 rvel;
  //Local2Rel (-STG3_OFF, vs.rpos);
  GlobalRot (_V(0,0,-0.5), rvel);
  vs.rvel = vs.rvel + rvel;
  vs.eng_main = vs.eng_hovr = 0.0;
  vs.status = 0;
  vs.vrot.x = vs.vrot.x;
  vs.vrot.z = vs.vrot.z;
  vs.vrot.y = vs.vrot.y;
  OBJHANDLE hStg2 = oapiCreateVessel ("Delta2_Stg2", "Delta2/delta2_stg2", vs);

  double main_trans = GetPropellantMass(hProp2);
  double rcs_trans = GetPropellantMass(hRCSProp2);
  VESSEL2* v =(VESSEL2*)oapiGetVesselInterface(hStg2);
  for (DWORD j = 0; j < 2; j++)
	{
	PROPELLANT_HANDLE ph = v->GetPropellantHandleByIndex (j);
	if(j == 0)v->SetPropellantMass(ph,main_trans);
	else if(j == 1)v->SetPropellantMass(ph,rcs_trans);
	}

	/*VESSELSTATUS2 vss;
	memset (&vss, 0, sizeof(vss));
	vss.version = 2;
	GetStatusEx(&vss);
	Local2Rel (STG3_OFF, vss.rpos);
	vss.vrot.z = vss.vrot.z + PI05;
	VECTOR3 rvel;
	GlobalRot (_V(0,0,0.5), rvel);
	vss.rvel = vss.rvel + rvel;
	DefSetStateEx (&vss);*/

    VECTOR3 angvel;
	GetAngularVel(angvel);
	angvel.z = angvel.z + PI05;
	SetAngularVel(angvel);

	DelMesh(0);
	eng_status = 2;

	CalcEmptyMass();
	ConfigMain2();

	PlayVesselWave(MyID, SEP, 0, 200, 25000);

}


void delta7926::JettisonYoyos (void)
{
VECTOR3 ang_vel;
GetAngularVel(ang_vel);
VECTOR3 yoyo1_vel_off = _V((8*ang_vel.z),0,0);
VECTOR3 yoyo2_vel_off = _V((-8*ang_vel.z),0,0);

VESSELSTATUS vs;
GetStatus(vs);
VECTOR3 rvel;
Local2Rel (YOYO1_OFF, vs.rpos);
GlobalRot (yoyo1_vel_off, rvel);
vs.rvel = vs.rvel + rvel;
vs.vrot.z = vs.vrot.z*2;
vs.status = 0;
oapiCreateVessel ("Delta_Yoyo_1", "Delta2/delta2_yoyo1", vs);

GetStatus(vs);
Local2Rel (YOYO2_OFF, vs.rpos);
GlobalRot (yoyo2_vel_off, rvel);
vs.rvel = vs.rvel + rvel;
vs.vrot.z = vs.vrot.z*2;
vs.status = 0;
oapiCreateVessel ("Delta_Yoyo_2", "Delta2/delta2_yoyo2", vs);
 
DelMesh(14);
yoyo_status = 3;
}


void delta7926::ConfigMain (void)
{
	DelThrusterGroup(htgrpmain,THGROUP_MAIN, true);
	DelThrusterGroup(htgrpbleft2,THGROUP_ATT_BANKLEFT, true);
	DelThrusterGroup(htgrpbright2,THGROUP_ATT_BANKRIGHT, true);
	DelThrusterGroup(htgrppup2,THGROUP_ATT_PITCHUP, true);
	DelThrusterGroup(htgrppdown2,THGROUP_ATT_PITCHDOWN, true);
	DelThrusterGroup(htgrpyleft2,THGROUP_ATT_YAWLEFT, true);
	DelThrusterGroup(htgrpyright2,THGROUP_ATT_YAWRIGHT, true);

	//DelPropellantResource(hProp1);
	SetPropellantMass(hProp1, 0);
	SetDefaultPropellantResource(hProp2);

	th_group[0] = th_rcs[14];
	htgrppup2 = CreateThrusterGroup (th_group, 1, THGROUP_ATT_PITCHUP);

	th_group[0] = th_rcs[15];
	htgrppdown2 = CreateThrusterGroup (th_group, 1, THGROUP_ATT_PITCHDOWN);

	th_group[0] = th_rcs[13];
	htgrpyright2 = CreateThrusterGroup (th_group, 1, THGROUP_ATT_YAWRIGHT);

	th_group[0] = th_rcs[12];
	htgrpyleft2 = CreateThrusterGroup (th_group, 1, THGROUP_ATT_YAWLEFT);

	th_group[0] = th_rcs[18];
	th_group[1] = th_rcs[19];
	htgrpbright2 = CreateThrusterGroup (th_group, 2, THGROUP_ATT_BANKRIGHT);

	th_group[0] = th_rcs[16];
	th_group[1] = th_rcs[17];
	htgrpbleft2 = CreateThrusterGroup (th_group, 2, THGROUP_ATT_BANKLEFT);
	
	th_group[0] = th_main[1];
	htgrpmain = CreateThrusterGroup (th_group, 1, THGROUP_MAIN);


	//SetAttachmentParams(pld_attach,_V( 0, 0, 7.05), _V( 0,0,1), _V(0,1,0));

	/*for (UINT i = 0; i < GetMeshCount(); i++)
	{
	ShiftMesh(i,-STG1STG2_OFF);
	}*/
	eng_light->SetPosition(_V(0,0,-3.5));
	SetSize (5);
	SetPMI(_V(9,9,1.9));
	SetCrossSections (_V(22,22,4));
	SetTouchdownPoints ( _V(0,-5, -10), _V(-5, 3, -10), _V( 5, 3,-10));
	SetCameraOffset (_V(0,2,0));
	ClearAirfoilDefinitions ();
	CreateAirfoil (LIFT_VERTICAL, _V(0,0,0), VLiftCoeff, 5, 10, 0.16);
	CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,0), HLiftCoeff, 5, 10, 0.16);
}

void delta7926::ConfigMain2 (void)
{
	DelThrusterGroup(htgrpmain,THGROUP_MAIN, true);
	DelThrusterGroup(htgrpbleft2,THGROUP_ATT_BANKLEFT, true);
	DelThrusterGroup(htgrpbright2,THGROUP_ATT_BANKRIGHT, true);
	DelThrusterGroup(htgrppup2,THGROUP_ATT_PITCHUP, true);
	DelThrusterGroup(htgrppdown2,THGROUP_ATT_PITCHDOWN, true);
	DelThrusterGroup(htgrpyleft2,THGROUP_ATT_YAWLEFT, true);
	DelThrusterGroup(htgrpyright2,THGROUP_ATT_YAWRIGHT, true);

	//DelPropellantResource(hProp1);
	SetPropellantMass(hProp1, 0);
	SetPropellantMass(hProp2, 0);
	SetPropellantMass(hRCSProp2, 0);
	SetDefaultPropellantResource(hProp3);

	th_group[0] = th_main[2];
	htgrpmain = CreateThrusterGroup (th_group, 1, THGROUP_MAIN);



	//ShiftMesh(7,-STG3_OFF);
	//ShiftMesh(8,-STG3_OFF);
	//if(has_adaptor == true)ShiftMesh(9,-STG3_OFF);

	//SetAttachmentParams (pld_attach[0], _V(0,0,0.906),_V(0,0,1), _V(0,1,0));
	//SetAttachmentParams (pld_attach[1], _V(0,0,0.906),_V(0,0,1), _V(0,1,0));
	//SetAttachmentParams (pld_attach[2], _V(0,0,0.906),_V(0,0,1), _V(0,1,0));

	ShiftCG(STG3_OFF);
	eng_light->SetPosition(_V(0,0,-1.5));
	SetSize (3);
	SetPMI(_V(4,4,4));
	SetCrossSections (_V(1,1,1));
	SetTouchdownPoints ( _V(0,-5, -1), _V(-5, 3, -1), _V( 5, 3,-1));
	SetCameraOffset (_V(0,2,0));
	ClearAirfoilDefinitions ();
	CreateAirfoil (LIFT_VERTICAL, _V(0,0,0), VLiftCoeff, 1, 1, 0.16);
	CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,0), HLiftCoeff, 1, 1, 0.16);
}

void delta7926::KillAllRCS2()
{
SetThrusterGroupLevel(THGROUP_ATT_PITCHUP, 0);
SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN, 0);
SetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT, 0);
SetThrusterGroupLevel(THGROUP_ATT_YAWLEFT, 0);
SetThrusterGroupLevel(THGROUP_ATT_BANKRIGHT, 0);
SetThrusterGroupLevel(THGROUP_ATT_BANKLEFT, 0);
}

void delta7926::CalcEmptyMass(void)
{
	double totalmass = STG3DRYMASS;
	
	if (srb_status == 0) totalmass = totalmass +  (SRB_DRYMASS*3);
	else if (srb_status == 1) totalmass = totalmass +  (SRB_DRYMASS*2);
	else if (srb_status == 2) totalmass = totalmass +  (SRB_DRYMASS*1);

	if (eng_status == 0) totalmass = totalmass + STG1DRYMASS + STG2DRYMASS;
	if (eng_status == 1) totalmass = totalmass + STG2DRYMASS;

	if (fair_status == 0) totalmass = totalmass + FAIR_MASS;

	
	OBJHANDLE hpld;
	pld_att_status = 0;

	hpld = GetAttachmentStatus(pld_attach[0]);
	if (hpld)
	{
		pld_mass1 = oapiGetMass(hpld);
		totalmass = totalmass + pld_mass1;
		oapiGetObjectName(hpld, pldname1, 30);
		pld_att_status = pld_att_status + 1;
	}
	else
	{
	pld_mass1 = 0;
	strcpy(pldname1, "NULL");
	}

	hpld = GetAttachmentStatus(pld_attach[1]);
	if (hpld)
	{
		pld_mass2 = oapiGetMass(hpld);
		totalmass = totalmass + pld_mass2;
		oapiGetObjectName(hpld, pldname2, 30);
		pld_att_status = pld_att_status + 1;
	}
	else
	{
	pld_mass2 = 0;
	strcpy(pldname2, "NULL");
	}

	hpld = GetAttachmentStatus(pld_attach[2]);
	if (hpld)
	{
		pld_mass3 = oapiGetMass(hpld);
		totalmass = totalmass + pld_mass3;
		oapiGetObjectName(hpld, pldname3, 30);
		pld_att_status = pld_att_status + 1;
	}
	else
	{
	pld_mass3 = 0;
	strcpy(pldname3, "NULL");
	}
	
	SetEmptyMass(totalmass);
}


void delta7926::DoAscent(void)
{
if (eng_status == 1)return;


CalcAzimuth();
CalcFlightVector();


double englev = GetThrusterGroupLevel(THGROUP_MAIN);

if(ap_function < 4)
{
if (tu_sw == 0)
	{
	OBJHANDLE hLpad = GetAttachmentStatus(lpd_attach);
	if (!hLpad) tu_sw = 1;
	else tu_sw = 2;
	}

if (tu_sw == 1) // not attached to launch pad, no throttle up
	{
	if (timer == 0) PlayVesselWave(MyID, CTDN, 0, 200, 0);
	timer = timer+step_time;

	if (timer > 66) 
	{
		englev = englev + (step_time*0.1);
		if (englev > 1)englev = 1;
	}
	else if (timer > 30) 
	{
		englev = englev - (step_time*0.1);
		if (englev < 0.8)englev = 0.8;
	}
	else if (timer > 10) englev = 1;
	else englev = 0;
	}
else if (tu_sw == 2) // attached to launch pad, do throttle up 2s
	{
	if (timer == 0) PlayVesselWave(MyID, CTDN, 0, 200, 0);
	timer = timer+step_time;
	
	if (timer > 66) 
	{
		englev = englev + (step_time*0.1);
		if (englev > 1)englev = 1;
	}
	else if (timer > 30) 
	{
		englev = englev - (step_time*0.1);
		if (englev < 0.8)englev = 0.8;
	}
	else if (timer > 10) englev = 1;
	else if (timer > 8) englev = englev + (step_time*0.5);
	else englev = 0;
	}
}
double th_max = GetThrusterMax(th_main[eng_status]);
double mass = GetMass();
double maxacc = th_max/mass;
double acc_lim = 60;
double eng_lim = acc_lim/maxacc;
if (eng_lim > 1)eng_lim = 1;
if (englev > eng_lim)englev = eng_lim;
SetThrusterGroupLevel(THGROUP_MAIN,englev);



if (GetAltitude() > 70 && ap_function == 0)
{
	ap_function = 1;
	//timer = 0;
}
if (ap_function > 0) RollToAz();
if (ap_function == 2) GoToFlightVector();
if (ap_function == 3) GoToFlightVector();
if (ap_function == 4) GoToFlightVector();
if (ap_function == 5)
	{
	KillAllRCS2();
	SetThrusterGroupLevel(THGROUP_MAIN,0);
	ActivateNavmode(NAVMODE_KILLROT);
	ap_function = 6;
	ap_status = 0;
	JettisonStg();
	op_status = 1;
	}




}

double delta7926::CalcAzimuth()
{
	double lon,lat,rad;
	GetEquPos(lon,lat,rad);
	if(abs(tgt_inc)<abs(lat)){
		tgt_inc=abs(lat);
	}else{
		tgt_inc=tgt_inc;
	}
	/////////////
	OBJHANDLE hearth=GetSurfaceRef();
	VECTOR3 relvel = _V(0,0,0);
	VECTOR3 globpos = _V(0,0,0);
	GetRelativeVel(hearth,relvel);
	GetGlobalPos(globpos);
	VECTOR3 relvel_glob = relvel+globpos;
	VECTOR3 rel_vel_loc = _V(0,0,0);
	Global2Local(relvel_glob,rel_vel_loc);

	VECTOR3 ov_hrz = _V(0,0,0);
	HorizonRot(rel_vel_loc, ov_hrz);

	double orb_ground_spd = sqrt((ov_hrz.x*ov_hrz.x)+(ov_hrz.z*ov_hrz.z));

	VECTOR3 orb_ground_vector = _V(ov_hrz.x,0,ov_hrz.z);

	////////
	double tgt_inc_sc = tgt_inc;
	if (GetAltitude(ALTMODE_GROUND) > 10000) // bloody winds at launch!!
	{
		if(lat > 0)
		{
		if (ov_hrz.z < -1 && tgt_inc > 0) tgt_inc_sc = -tgt_inc;
		}
		else if(lat < 0)
		{
		if (ov_hrz.z > 1 && tgt_inc < 0) tgt_inc_sc = -tgt_inc;
		}
	}

	double final_azimuth = asin(cos(tgt_inc_sc)/cos(lat));
	final_azimuth+=(90*RAD-final_azimuth)*(1-(tgt_inc_sc/abs(tgt_inc_sc)));
	VECTOR3 final_ground_vector = _V((FinalV()*sin(final_azimuth)),0,(FinalV()*cos(final_azimuth)));

	VECTOR3 req_ground_vector = final_ground_vector - orb_ground_vector;

	azimuth = atan2(req_ground_vector.x,req_ground_vector.z);

	/////////////
	//azimuth=atan2(FinalV()*(sin(asin(cos(tgt_inc)/cos(lat))))-GetOS(),FinalV()*(cos(asin(cos(tgt_inc)/cos(lat)))));
	//azimuth+=(90*RAD-azimuth)*(1-(tgt_inc/abs(tgt_inc)));
	
	vazim1=_V(sin(azimuth),0,cos(azimuth));
	
	HorizonInvRot(vazim1,azim_local);
	//sprintf(oapiDebugString(), "ap_function %d", ap_function);
	return azimuth;
}
double delta7926::GetOS(){
	OBJHANDLE hearth=GetSurfaceRef();
	VECTOR3 relvel;
	GetRelativeVel(hearth,relvel);
	double os=length(relvel);
	return os;
}

double delta7926::FinalV()
{
//double tgtapo = 180000;
//double tgtperi = -6318000;
double tgtapo = 200000;
double tgtperi = 2000000;
double abside=tgtapo;
	if(tgtapo<=260000){
			abside=tgtapo;
		}else{
			abside=tgtperi;}
if (op_status == 1) return sqrt(mu*(2/(op_meco_alt+rt)-1/(0.5*(op_tgt+op_meco_alt+(2*rt)))));
else return sqrt(mu*(2/(abside+rt)-1/(0.5*(tgtapo+tgtperi+2*rt))));

}

void delta7926::CalcFlightVector()
{
	
	double plf = -0.25*(cos(tgt_inc));

	double apof = 0.5+(1.5*((ap_tgt - 200000)/100000));

	plf = plf + apof;

	double initial_pitch = (74.0+plf)*RAD;
	if (initial_pitch > (88.5*RAD))initial_pitch = (88.5*RAD);////////////////

	if (ap_function == 2) 
	{
		VECTOR3 haspdv = _V(0,1,0);
		GetGroundspeedVector (FRAME_HORIZON, haspdv);
		double mass = GetMass();
		VECTOR3 thrustvector = _V(0,0,0);
		GetThrustVector(thrustvector);
		double trst = length(thrustvector);
		double acc = trst/mass;
		double time_avail = (180-length(haspdv))/(acc-9.8);
		double pitch_rate = ((pitch_angle)-initial_pitch)/time_avail;
		pitch_angle = pitch_angle - (step_time*pitch_rate);
		if(time_avail < 0.1)pitch_angle = initial_pitch;

		//sprintf(oapiDebugString(), "initial_pitch %0.4f pitch_angle %0.4f", initial_pitch*DEG,pitch_angle*DEG);

		if (length(haspdv) > 180 && GetAOA() > 0)ap_function = 3;
	}
	
	
	else if (ap_function == 3)
	{
		VECTOR3 haspdv = _V(0,1,0);
		GetGroundspeedVector (FRAME_HORIZON, haspdv);
		double xzlngth = sqrt((haspdv.x*haspdv.x)+(haspdv.z*haspdv.z));
		if (haspdv.y < 0)haspdv.y = 0;
		pitch_angle = atan2(haspdv.y,xzlngth);
		if (GetAltitude() > 40000 && srb_status == 3) 
		{
			timer = 0;
			ap_function = 4;
		}
	}

	else if (ap_function == 4)
	{
		double ap_dist = 0;
		GetApDist(ap_dist);

		double englev = GetThrusterGroupLevel(THGROUP_MAIN);
		double fuel_remaining = GetPropellantMass(hProp1);
		double isp_curr = GetThrusterIsp(th_main[eng_status]);
		double th_max = GetThrusterMax(th_main[eng_status]);
		double fuel_rate = englev*th_max/isp_curr;
		double mintime = fuel_remaining/fuel_rate;
		if (mintime < 0)mintime = 0;

		/////////////////////////

		VECTOR3 haspdv = _V(0,1,0);
		//GetHorizonAirspeedVector(haspdv);
		GetGroundspeedVector (FRAME_HORIZON, haspdv);
		//double xzlngth = sqrt((haspdv.x*haspdv.x)+(haspdv.z*haspdv.z));
		if (haspdv.y < 0)haspdv.y = 0;

		///////////////////////////////////////////////////////
		double vmass = GetMass();

		OBJHANDLE hearth=GetSurfaceRef();
		VECTOR3 relvel = _V(0,0,0);
		GetRelativeVel(hearth,relvel);
		double orb_vel = length(relvel);
		double orb_grnd_spd = sqrt((orb_vel*orb_vel)-(haspdv.y*haspdv.y));
		double circ = PI2*(rt+GetAltitude());
		double ang_vel = PI2*orb_grnd_spd/circ;
		VECTOR3 angmomvect = _V(0,(-1*ang_vel),0);
		VECTOR3 velvect = _V(0,0,orb_grnd_spd);
		VECTOR3 corvect = crossp(angmomvect,velvect);
		double acc_cor = length(corvect);

		VECTOR3 zed_axis = _V(0,0,1);
		VECTOR3 zed_axis_hrz = _V(0,0,0);
		HorizonRot(zed_axis, zed_axis_hrz);
		double zed_axis_hrz_comp = sqrt((zed_axis_hrz.x*zed_axis_hrz.x)+(zed_axis_hrz.z*zed_axis_hrz.z));
		double current_pitch = atan2(zed_axis_hrz.y, zed_axis_hrz_comp);

		double cosp = cos(current_pitch);
		double dv_grnd = isp_curr*log(vmass/(vmass-fuel_remaining));
		dv_grnd = dv_grnd*cosp;

		double final_orb_grnd_spd = orb_grnd_spd+dv_grnd;
		velvect = _V(0,0,final_orb_grnd_spd);
		VECTOR3 final_corvect = crossp(angmomvect,velvect);
		double final_acc_cor = length(corvect);

		VECTOR3 weight = _V(0,0,0);
		GetWeightVector(weight);
		
		double gravt = length(weight);
		double gg = (gravt/vmass)-final_acc_cor;
		double aa = (mintime*mintime)/(2*gg);
		double bb = (haspdv.y*mintime/gg)+((mintime*mintime)/2);
		double cc = -(ap_tgt - GetAltitude() - (haspdv.y*mintime) - ((haspdv.y*haspdv.y)/(2*gg)) );

		double req_acc_1 = (-bb+(sqrt((bb*bb)-(4*aa*cc))))/(2*aa);
		double req_acc_2 = (-bb-(sqrt((bb*bb)-(4*aa*cc))))/(2*aa);

		//if (mintime < 4) englev = englev-(step_time/5);
		//if(englev < 0.1)englev = 0.1;

		double curr_acc = th_max*englev/(vmass-(fuel_remaining*0.125));
		
		//if (ap_dist > (rt+ap_tgt))ap_function = 5;
		//if (fuel_remaining < (0.5*fuel_rate*step_time))ap_function = 5;
		if (fuel_remaining == 0)ap_function = 5;
		
		double total_req_acc = req_acc_1+gg-acc_cor;

		double psc = total_req_acc/curr_acc;
		//if (psc > 0.985)psc = 0.985;
		if (psc > 0.9962)psc = 0.9962;
		else if (psc < -0.9962)psc = -0.9962;
		double pangle = asin(psc);

		if (mintime < 2)pitch_angle = current_pitch;
		else pitch_angle = pangle;

		
		//sprintf(oapiDebugString(), "th_max %0.4f ", th_max);

	}

		
		vazim2 = _V(vazim1.x, (tan(pitch_angle)), vazim1.z);

		if (ap_function < 2)
		{
		VECTOR3 haspdv = _V(0,1,0);
		GetGroundspeedVector (FRAME_HORIZON, haspdv);
		vazim2 = _V(-haspdv.x*0.5,haspdv.y,-haspdv.z*0.5);
		}

		HorizonInvRot(vazim2,flight_vector);

		//sprintf(oapiDebugString(), "af %d", ap_function);
}

void delta7926::RollToAz()
{
DeactivateNavmode(NAVMODE_KILLROT);
if(ap_function > 3)HorizonInvRot(_V(0,-1,0),azim_local);

double roll_angle = atan2(azim_local.x,-azim_local.y);
if (roll_angle > PI) roll_angle = roll_angle - PI2;
else if (roll_angle < -PI) roll_angle = roll_angle + PI2;

	double mass = GetMass();

	double max_rcs = GetThrusterMax(th_rcs[0]);
	double max_torque = 2*max_rcs*4;
	double inertia = 1.9*mass;

	if (eng_status == 1)
	{
	max_rcs = GetThrusterMax(th_rcs[19]);
	max_torque = 2*max_rcs*0.73;
	inertia = 1.9*mass;
	}
	VECTOR3 avel = _V(0,0,0);
	GetAngularVel(avel);


	double max_acc = max_torque/inertia;

	double max_drot = max_acc*step_time;

	double reqzv = 0.9*(sqrt(abs(roll_angle*max_acc*2)));
	if (roll_angle > 0)reqzv = -reqzv;

	double req_drot = reqzv - avel.z;

	double thz_lvl = req_drot/max_drot;


	if((roll_angle*DEG) < 1 && (roll_angle*DEG) > -1)
	{
	reqzv = 1*-roll_angle;
	double acc_reqz = (reqzv-avel.z)/0.1;
	thz_lvl = acc_reqz/max_acc;
	}


	if(thz_lvl > 1)thz_lvl = 1;
	else if(thz_lvl < -1)thz_lvl = -1;


	if(thz_lvl > 0)
	{
		SetThrusterGroupLevel(THGROUP_ATT_BANKRIGHT, thz_lvl);
		SetThrusterGroupLevel(THGROUP_ATT_BANKLEFT, 0);
	}
	else if(thz_lvl < 0)
	{
		SetThrusterGroupLevel(THGROUP_ATT_BANKLEFT, -thz_lvl);
		SetThrusterGroupLevel(THGROUP_ATT_BANKRIGHT, 0);
	}
	else 
	{
		SetThrusterGroupLevel(THGROUP_ATT_BANKRIGHT, 0);
		SetThrusterGroupLevel(THGROUP_ATT_BANKLEFT, 0);
	}

	if (ap_function == 1)
	{

		if (roll_angle > -0.1 && roll_angle < 0.1)
		{
		VECTOR3 aspdv = _V(0,1,0);
		GetHorizonAirspeedVector(aspdv);
		//double azimuth = CalcAzimuth();
		//double az_factor = 30*(1 - sin(azimuth));
		if (aspdv.y > (50)) //+az_factor
			{
			vrot_lim = 2*RAD;
			ap_function = 2;
			}
		}
	}
}



void delta7926::GoToFlightVector()
{
	//VECTOR3 wv = _V(0,0,0);
	//GetWeightVector(wv);
	//wv = -wv;

	
	//double angoa = atan2(-wv.y, wv.z);
	//double slipage = atan2(-wv.x, wv.z);
	DeactivateNavmode(NAVMODE_KILLROT);

	double angoa = atan2(-flight_vector.y, flight_vector.z);
	double slipage = atan2(flight_vector.x, flight_vector.z);

	if(angoa > PI)angoa = (PI*2)-angoa;
	else if(angoa < -PI)angoa = (PI*2)+angoa;

	if(slipage > PI)slipage = (PI*2)-slipage;
	else if(slipage < -PI)slipage = (PI*2)+slipage;

	double mass = GetMass();

	double max_rcs = GetThrusterMax(th_rcs[0]);
	double max_torque_x = 2*max_rcs*20;
	double max_torque_y = 2*max_rcs*20;
	double inertia = 60*mass;
	
	if (eng_status == 1)
	{
	max_rcs = GetThrusterMax(th_rcs[13]);
	max_torque_x = 1*max_rcs*3;
	max_torque_y = 1*max_rcs*3;
	inertia = 9*mass;
	if (rcs_sw2 == 1)
		{
		max_rcs = GetThrusterMax(th_rcs[16]);
		max_torque_x = 2*max_rcs*0.4195;
		max_torque_y = 2*max_rcs*0.398;
		}
	}

	VECTOR3 avel = _V(0,0,0);
	GetAngularVel(avel);
	double max_acc_x = max_torque_x/inertia;
	double max_drot_x = max_acc_x*step_time;

	double reqxv = 0.9*(sqrt(abs(angoa*max_acc_x*2)));
	if (reqxv > vrot_lim)reqxv = vrot_lim;
	if (angoa > 0)reqxv = -reqxv;
	double req_drotx = reqxv - avel.x;
	double thx_lvl = req_drotx/max_drot_x;
	if((angoa*DEG) < 1 && (angoa*DEG) > -1)
	{
	reqxv = 1*-angoa;
	double acc_reqx = (reqxv-avel.x)/0.1;
	thx_lvl = acc_reqx/max_acc_x;
	}

	double max_acc_y = max_torque_y/inertia;
	double max_drot_y = max_acc_y*step_time;

	double reqyv = 0.9*(sqrt(abs(slipage*max_acc_y*2)));
	if (reqyv > vrot_lim)reqyv = vrot_lim;
	if (slipage > 0)reqyv = -reqyv;
	double req_droty = reqyv - avel.y;
	double thy_lvl = req_droty/max_drot_y;
	if((slipage*DEG) < 1 && (slipage*DEG) > -1)
	{
	reqyv = 1*-slipage;
	double acc_reqy = (reqyv-avel.y)/0.1;
	thy_lvl = acc_reqy/max_acc_y;
	}

	//sprintf(oapiDebugString(), "slip %0.4f yrot %0.4f reqyv%0.4f", (slipage*DEG),(avel.y*DEG), (reqyv*DEG));


	if(thx_lvl > 1)thx_lvl = 1;
	else if(thx_lvl < -1)thx_lvl = -1;

	if(thy_lvl > 1)thy_lvl = 1;
	else if(thy_lvl < -1)thy_lvl = -1;


	if(thx_lvl > 0)
		{
		SetThrusterGroupLevel(THGROUP_ATT_PITCHUP, thx_lvl);
		SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN, 0);
		}
	else if(thx_lvl < 0)
	{
		SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN, -thx_lvl);
		SetThrusterGroupLevel(THGROUP_ATT_PITCHUP, 0);
	}
	else 
	{
		SetThrusterGroupLevel(THGROUP_ATT_PITCHUP, 0);
		SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN, 0);
	}


	if(thy_lvl > 0)
		{
		SetThrusterGroupLevel(THGROUP_ATT_YAWLEFT, thy_lvl);
		SetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT, 0);
		}
	else if(thy_lvl < 0)
		{
		SetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT, -thy_lvl);
		SetThrusterGroupLevel(THGROUP_ATT_YAWLEFT, 0);
		}
	else 
	{
		SetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT, 0);
		SetThrusterGroupLevel(THGROUP_ATT_YAWLEFT, 0);
	}


}

bool delta7926::clbkLoadGenericCockpit ()
{
	SetCamDir();
return true;
}

void delta7926::SetCamDir()
{
	VECTOR3 camdefdir = _V(0,0,1);
	if(cam_status == 1)camdefdir = _V(0,0,-1);
	SetCameraDefaultDirection(camdefdir, 0);
	oapiCameraAttach(GetHandle(), 2);
}

void delta7926::DoOrbit()
{
	
	
	if (start_sw == 0)
	{
	timer = timer+step_time;
	if (timer < 5) return;
	else
		{
		start_sw = 1;
		timer = 0;
		SetThrusterLevel(th_main[1],1);
		}
	}

	else if (start_sw == 1 && fair_status == 0)
	{
	timer = timer+step_time;
	if (timer > 20)
		{
			JettisonFair();
			timer = 0;
		}
	}


DeactivateNavmode(NAVMODE_KILLROT);
CalcAzimuth();

double fnl_vel = sqrt(mu*(2/(op_meco_alt+rt)-1/(0.5*(op_tgt+op_meco_alt+(2*rt)))));

double emp_mass = GetEmptyMass();
double vmass = GetMass();

OBJHANDLE hearth=GetSurfaceRef();

VECTOR3 haspdv = _V(0,0,0);
GetGroundspeedVector(FRAME_HORIZON, haspdv);
	
VECTOR3 weight = _V(0,0,0);
GetWeightVector(weight);

//centrifugal acc
	VECTOR3 relvel = _V(0,0,0);
	GetRelativeVel(hearth,relvel);
	double orb_vel = length(relvel);
	double orb_grnd_spd = sqrt((orb_vel*orb_vel)-(haspdv.y*haspdv.y));
	double rds = (rt + GetAltitude());
	double ang_vel = orb_grnd_spd/rds;
	double acc_cent_fact = 1;
	double acc_cent = ang_vel*ang_vel*rds*acc_cent_fact;

	//double rds_fnl = (rt + op_meco_alt);
	//double ang_vel_fnl = fnl_vel/rds_fnl;
	//double acc_cent_fact_fnl = 1;
	//double acc_cent_fnl = ang_vel_fnl*ang_vel_fnl*rds_fnl*acc_cent_fact_fnl;

		
	double gravt = length(weight);
	double g = -(gravt/vmass)+acc_cent;

	double aa = g/2;
	double bb = haspdv.y;
	double cc = GetAltitude()-op_meco_alt;

	//double timetoimp = (-bb-(sqrt((bb*bb)-(4*aa*cc))))/(2*aa);

	double max_th = GetThrusterMax(th_main[1]);
	double max_acc = max_th/vmass;
	double acc_lim = 50;
	double max_eng_lvl = 1;
	if (max_acc > acc_lim) max_eng_lvl = acc_lim/max_acc;
	if (max_eng_lvl < 0.7)max_eng_lvl = 0.7;
	if ((fnl_vel-orb_vel) < 30)
	{
	if (max_eng_lvl > 0.7)max_eng_lvl = 0.7;
	}

	VECTOR3 zed_axis = _V(0,0,1);
	VECTOR3 zed_axis_hrz = _V(0,0,0);
	HorizonRot(zed_axis, zed_axis_hrz);
	double zed_axis_hrz_comp = sqrt((zed_axis_hrz.x*zed_axis_hrz.x)+(zed_axis_hrz.z*zed_axis_hrz.z));
	double current_pitch = atan2(zed_axis_hrz.y, zed_axis_hrz_comp);

	double sinp = sin(current_pitch);
	double cosp = cos(current_pitch);
	
	double dv_req = (fnl_vel - orb_grnd_spd)/cosp;
	double fraction = exp(dv_req/GetThrusterIsp(th_main[1]));
	double fuelmass = vmass - (vmass/fraction);

	double dth = step_time * 0.2;

	double englevel = GetThrusterLevel(th_main[eng_status]);
	
	
	double fuelrate = GetThrusterMax(th_main[1])*englevel/GetThrusterIsp(th_main[1]);
	double dvtime = fuelmass/fuelrate;
	//dvtime = dvtime/cosp;

	GetElements (hearth,el,&prm,0,FRAME_EQU);
	double ap_time = prm.ApT;
	//sprintf(oapiDebugString(), "dvtime %0.1f ap_time %0.1f", dvtime, ap_time);
	double ap_time_limiter = dvtime/ap_time;
	if (ap_time_limiter > 1)ap_time_limiter = 1;
	else if (ap_time_limiter < 0.7)ap_time_limiter = 0.7;
	if(haspdv.y < 0)ap_time_limiter = 1;
	if (ap_time_limiter < max_eng_lvl) max_eng_lvl = ap_time_limiter;
	

	double vvel_fnl = 0;

	//double acc_req = 2*((vvel_fnl*dvtime)+GetAltitude()-op_meco_alt)/(dvtime*dvtime);
	//double acc_req = 2*2*(op_meco_alt-GetAltitude()-(haspdv.y*dvtime))/(dvtime*dvtime);
	double acc_req = ((6*(op_meco_alt-GetAltitude() - (haspdv.y*dvtime)))+(2*dvtime*haspdv.y))/(dvtime*dvtime);
	
	acc_req = acc_req - g;

	//sprintf(oapiDebugString(), "acc_req %0.1f", acc_req);

	double srp = acc_req/(max_acc*englevel);
	if (srp > 1)srp = 1;
	else if (srp < -1)srp = -1;
	double req_pitch = asin(srp);
	double crp = cos(req_pitch);

	VECTOR3 fv2d = _V(0, srp, crp);
	VECTOR3 fv_hrz = _V(0,0,0);
	fv_hrz.x = sin(azimuth)*crp;
	fv_hrz.y = srp;
	fv_hrz.z = cos(azimuth)*crp;

	double max_dv = step_time*max_acc*englevel;
	double req_englevel = (fnl_vel-orb_vel)/max_dv;
	if (req_englevel < 0)
	{
		op_status = 0;
		englevel = 0;
		KillAllRCS2();
		SetThrusterLevel(th_main[1],0);
		ActivateNavmode(NAVMODE_KILLROT);
		return;
	}
	//englevel = (fnl_vel-orb_vel)/max_dv;
	if (req_englevel > max_eng_lvl)req_englevel = max_eng_lvl;
	

	if (englevel > req_englevel)
	{
		englevel = englevel-dth;
		if (englevel < req_englevel) englevel = req_englevel;
	}
	else if (englevel < req_englevel)
	{
		englevel = englevel+dth;
		if (englevel > req_englevel) englevel = req_englevel;
	}
	

	if ((fnl_vel-orb_vel) < 30)
	{
		//flight_vector = flight_vector;
		ActivateNavmode(NAVMODE_KILLROT);
		///if(englevel > 0)englevel = 0.7;
		///if (englevel > max_eng_lvl)englevel = max_eng_lvl;
		//sprintf(oapiDebugString(), "fv-ov %0.1f", (fnl_vel-orb_vel));
	}

	//else if ((fnl_vel-orb_vel) < 400)
	//{
	//	if(englevel > 0)englevel = 0.7;
	//	if (englevel > max_eng_lvl)englevel = max_eng_lvl;
	//	HorizonInvRot(fv_hrz, flight_vector);
	//	GoToFlightVector();
	//}
	else 
	{
	HorizonInvRot(fv_hrz, flight_vector);
	GoToFlightVector();
	}

	if (start_sw == 0)englevel = 0;
	SetThrusterLevel(th_main[1],englevel);




}