// ==============================================================
//                 ORBITER MODULE: slc17
//
// slc17.cpp
// Control module for slc17 vessel class
//
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"
#include "gcAPI.h"

// ==============================================================
// Some vessel parameters
// ==============================================================

const double ANTENNA_OPERATING_SPEED = 0.8;
const double CHUTE_OPERATING_SPEED = 0.01;
const double PB_FUELMASS = 750.0;
const double PB_ISP = 5e4;
const double PB_MAXHOVERTH = 0;
const VECTOR3 MSS_OFF = _V(-181.535,0,0);
const double x_off = -181.535;

static const DWORD ntdvtx_geardown = 7;
static TOUCHDOWNVTX tdvtx_geardown[ntdvtx_geardown] = {
	{_V(   0, -1.50,100), 1e6, 1e5, 2, 2},
	{_V(-86.6,-1.50,-50), 1e6, 1e5, 2, 2},
	{_V( 86.6,-1.50,-50), 1e6, 1e5, 2, 2},
	{_V(   0,  50,   0), 1e5, 1e6, 2},
	{_V(    0, -1.50,-100), 1e6, 1e5, 2},
	{_V(-86.6,-1.50, 50), 1e6, 1e5, 2},
	{_V( 86.6,-1.50, 50), 1e6, 1e5, 2},
};

// Calculate lift coefficient [Cl] as a function of aoa (angle of attack) over -Pi ... Pi
// Implemented here as a piecewise linear function
class slc17: public VESSEL3 {
public:
	slc17 (OBJHANDLE hVessel, int flightmodel)
		: VESSEL3 (hVessel, flightmodel) {}
	void DefineAnimations (void);
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPreStep(double SimT, double SimDT, double MJD);
	//void clbkPostStep (double simt, double simdt, double mjd);
	void clbkPostCreation();
	void clbkSaveState (FILEHANDLE scn);
	void clbkLoadStateEx (FILEHANDLE scn, void *status);
	int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);

	


private:
	THRUSTER_HANDLE th_hover, th_steam, th_hovera, th_steama;
	ATTACHMENTHANDLE hAtt_A, hAtt_B;
	UINT anim_ant, anim_chute, anim_antb, anim_chuteb;
	double ant_proc, chute_proc, ant_procb, chute_procb, timer;
	int ant_status, chute_status, ant_statusb, chute_statusb;
	int light_status, vent_status, launch_status;
	SpotLight *slca1_light;
	SpotLight *slca2_light;
	SpotLight *slcb1_light;
	SpotLight *slcb2_light;
};
// Define animation sequences for moving parts
// --------------------------------------------------------------
void slc17::DefineAnimations (void)
{
	// 1. UMBILICALS
	anim_ant = CreateAnimation (0);
	static UINT HiGainAnt1Grp[3] = {7,8,9};

	static MGROUP_ROTATE  HiGainAnt1 (1, HiGainAnt1Grp, 3, _V(3.125,0,3.915), _V(0,1,0), (float)(PI*0.611));
	AddAnimationComponent (anim_ant, 0.2, 1,   &HiGainAnt1);



	// 2. TOWER AND COWLS
	anim_chute = CreateAnimation (0);
	static UINT ChuteGrp[35] = {12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46};

	static MGROUP_TRANSLATE  ChuteFold1 (1, ChuteGrp, 35, _V(0,0,-110));
	AddAnimationComponent (anim_chute, 0, 1,   &ChuteFold1);

	/////////////// MSS B

	// 1. UMBILICALS
	anim_antb = CreateAnimation (0);
	static UINT HiGainAnt1Grpb[3] = {8,9,10};

	static MGROUP_ROTATE  HiGainAnt1b (2, HiGainAnt1Grpb, 3, _V(3.125,0,3.915), _V(0,1,0), (float)(PI*0.611));
	AddAnimationComponent (anim_antb, 0.2, 1,   &HiGainAnt1b);



	// 2. TOWER AND COWLS
	anim_chuteb = CreateAnimation (0);
	static UINT ChuteGrpb[35] = {13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47};
	static UINT ChuteGrp1b[1] = {0};

	static MGROUP_TRANSLATE  ChuteFold1b (2, ChuteGrpb, 35, _V(0,0,-110));
	AddAnimationComponent (anim_chuteb, 0, 0.7,   &ChuteFold1b);

	static MGROUP_TRANSLATE  ChuteFold2b (2, ChuteGrp1b, 1, _V(0,0,-18.9));
	AddAnimationComponent (anim_chuteb, 0.71, 1,   &ChuteFold2b);

}


// ==============================================================
// slc17 class interface
// ==============================================================

// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void slc17::clbkSetClassCaps (FILEHANDLE cfg)
{
	launch_status = 0;
	timer = 0;
	ant_proc = 0.0;
	ant_status = 0;
	chute_proc = 0.0;
	chute_status = 0;
	ant_procb = 0.0;
	ant_statusb = 0;
	chute_procb = 0.0;
	chute_statusb = 0;
	DefineAnimations ();
	light_status = 0;
	vent_status = 0;

	gcInitialize();

	// physical specs
	SetEmptyMass (1000000);
	SetCW (1, 1, 1, 1);
	SetSize (200);
	SetVisibilityLimit(0.02);
	//SetCOG_elev (2.47);
	SetCrossSections (_V(153,187,126));
	SetPMI (_V(15,22,8));
	//SetTouchdownPoints (_V(0,0.01,9), _V(-4,0.01,-9), _V(4,0.01,-9));
	SetTouchdownPoints (tdvtx_geardown,ntdvtx_geardown);
	SetRotDrag (_V(1,1,1));

	SetMeshVisibilityMode ((AddMesh ("SLC17/slc17_base")), MESHVIS_ALWAYS);
	SetMeshVisibilityMode ((AddMesh ("SLC17/slc17_mss_a", &MSS_OFF)), MESHVIS_ALWAYS);
	SetMeshVisibilityMode ((AddMesh ("SLC17/slc17_mss_b")), MESHVIS_ALWAYS);
	SetCameraOffset (_V(-90,15,-210));

	PROPELLANT_HANDLE ph_vernier = CreatePropellantResource (PB_FUELMASS);

	PARTICLESTREAMSPEC contrail_hover = {
		0, 0.2, 20, 0.5, 10, 1.5, 1, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_PSQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	SURFHANDLE smokeD3D9 = oapiRegisterParticleTexture ("SLC17/slc17_smoke");

	PARTICLESTREAMSPEC contrail_steam = {
		0, 6, 20, 50, 1.75, 3, 10, 2, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_PLIN, 0.25, 0.75,
		PARTICLESTREAMSPEC::ATM_PLIN, 1e-4, 1,
		smokeD3D9
	};

	if (gcEnabled() == true)
	{
		//contrail_steam.srcrate = 30;
		contrail_steam.srcsize = 12;
		contrail_steam.lifetime = 4;
		contrail_hover.srcsize = 0.5;
		contrail_hover.srcspread = 5;
		contrail_hover.atmslowdown = 0.5;
		contrail_hover.v0 = 0.6;
		contrail_hover.lifetime = 2;
	}

	th_hover = CreateThruster (_V(0,0,0), _V(0,1,0), PB_MAXHOVERTH, ph_vernier, PB_ISP);
	th_hovera = CreateThruster (_V(x_off,0,0), _V(0,1,0), PB_MAXHOVERTH, ph_vernier, PB_ISP);

	th_steam = CreateThruster (_V(0,0,0), _V(0,-1,0), PB_MAXHOVERTH, ph_vernier, PB_ISP);
	th_steama = CreateThruster (_V(x_off,0,0), _V(0,-1,0), PB_MAXHOVERTH, ph_vernier, PB_ISP);


	AddExhaustStream (th_hover, _V(1.27,25.08, 0), &contrail_hover);
	AddExhaustStream (th_hovera, _V((1.27 + x_off),25.08, 0), &contrail_hover);

	AddExhaustStream (th_steam, _V(0,-2,0), &contrail_steam);
	AddExhaustStream (th_steama, _V(x_off,-2,0), &contrail_steam);


	// ***************** lights *******************

	COLOUR4 col_d = {1,1,0.9,0};
	COLOUR4 col_s = {1,1,0.9,0};
	COLOUR4 col_a = {0,0,0,0};
	COLOUR4 col_white = {1,1,1,0};
	//slca1_light = (SpotLight*)AddSpotLight(_V(5,45.275,3), _V(-0.3,-0.9055,-0.3), 70, 5e-3, 0, 5e-3, RAD*30, RAD*60, col_a, col_a, col_a);
	slca1_light = (SpotLight*)AddSpotLight(_V(-17,15,17), _V(0.612,0.5,-0.612), 500, 8e-4, 0, 8e-4, RAD*45, RAD*100, col_d, col_s, col_a);
	slca1_light->Activate(false);

	slca2_light = (SpotLight*)AddSpotLight(_V(0,15,-25), _V(0,0.5,0.866), 500, 7.5e-4, 0, 7.5e-4, RAD*45, RAD*100, col_d, col_s, col_a);
	slca2_light->Activate(false);

	slcb1_light = (SpotLight*)AddSpotLight(_V((-17+x_off),15,17), _V(0.612,0.5,-0.612), 500, 8e-4, 0, 8e-4, RAD*45, RAD*100, col_d, col_s, col_a);
	slcb1_light->Activate(false);

	slcb2_light = (SpotLight*)AddSpotLight(_V((0+x_off),15,-25), _V(0,0.5,0.866), 500, 7.5e-4, 0, 7.5e-4, RAD*45, RAD*100, col_d, col_s, col_a);
	slcb2_light->Activate(false);

	//slca3_light = (SpotLight*)AddSpotLight(_V(-17,15,17), _V(0.612,0.5,-0.612), 500, 8e-4, 0, 8e-4, RAD*45, RAD*100, col_d, col_s, col_a);
	//slca3_light->Activate(false);

	////////////////// att points /////////////////////

	hAtt_A = CreateAttachment(false, _V(x_off,8.26,0), _V(0,1,0), _V(0,0,1), "17A", false);
	hAtt_B = CreateAttachment(false, _V(0,8.26,0), _V(0,1,0), _V(0,0,1), "17B", false);

}

// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new slc17 (hvessel, flightmodel);
}
// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (slc17*)vessel;
}
// ==============================================================
// Overloaded clbkPostCreation callback functions
// ==============================================================

void slc17::clbkPostCreation ()
{
}
void slc17::clbkSaveState (FILEHANDLE scn)
{
	char cbuf[256];

	// default vessel parameters
	VESSEL3::clbkSaveState (scn);

	// custom parameters
	sprintf (cbuf, "%d %0.4f %d %0.4f", ant_status, ant_proc, ant_statusb, ant_procb);
	oapiWriteScenario_string (scn, "UMB", cbuf);
	sprintf (cbuf, "%d %0.4f %d %0.4f", chute_status, chute_proc, chute_statusb, chute_procb);
	oapiWriteScenario_string (scn, "TOWER", cbuf);
	sprintf (cbuf, "%d %0.4f", launch_status, timer);
	oapiWriteScenario_string (scn, "TIMR", cbuf);
}
void slc17::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
    char *line;
	while (oapiReadScenario_nextline (scn, line)) {
		if (!strnicmp (line, "UMB", 3)) {
			sscanf (line+3, "%d%lf%d%lf", &ant_status, &ant_proc, &ant_statusb, &ant_procb);
		} else if (!strnicmp (line, "TOWER", 5)) {
			sscanf (line+5, "%d%lf%d%lf", &chute_status, &chute_proc, &chute_statusb, &chute_procb);
		} else if (!strnicmp (line, "TIMR", 4)) {
			sscanf (line+4, "%d%lf", &launch_status, &timer);
		}else {
            ParseScenarioLineEx (line, vs);
			// unrecognised option - pass to Orbiter's generic parser
        }
   }
	SetAnimation (anim_ant, ant_proc);
	SetAnimation (anim_chute, chute_proc);
	SetAnimation (anim_antb, ant_procb);
	SetAnimation (anim_chuteb, chute_procb);
}


int slc17::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate) 
{
  if (!down) return 0; // only process keydown events

  if (KEYMOD_SHIFT (kstate)) 
	{
		switch (key)
		{
		case OAPI_KEY_G: // move mss B
		if(chute_statusb == 0)chute_statusb = 1;
		else if(chute_statusb == 1)chute_statusb = 3;
		else if(chute_statusb == 2)chute_statusb = 3;
		else if(chute_statusb == 3)chute_statusb = 1;
		return 1;

		case OAPI_KEY_B: // move umbilicals B
		if(ant_statusb == 0)ant_statusb = 1;
		else if(ant_statusb == 1)ant_statusb = 3;
		else if(ant_statusb == 2)ant_statusb = 3;
		else if(ant_statusb == 3)ant_statusb = 1;
		return 1;
		}
	}
  else 
	{ // unmodified keys
    switch (key) 
		{
	case OAPI_KEY_G: // move mss A
		if(chute_status == 0)chute_status = 1;
		else if(chute_status == 1)chute_status = 3;
		else if(chute_status == 2)chute_status = 3;
		else if(chute_status == 3)chute_status = 1;
		return 1;
	case OAPI_KEY_B: // move umbilicals A
		if(ant_status == 0)ant_status = 1;
		else if(ant_status == 1)ant_status = 3;
		else if(ant_status == 2)ant_status = 3;
		else if(ant_status == 3)ant_status = 1;
		return 1;
	case OAPI_KEY_K: // lights on/off
		if (light_status == 0) light_status = 1;
		else if (light_status == 1) light_status = 0;
		return 1;
	case OAPI_KEY_V: // LOX vent on/off
		if (vent_status == 0) vent_status = 1;
		else if (vent_status == 1) vent_status = 0;
		return 1;
		}
	}
  return 0;
}
void slc17::clbkPreStep(double SimT, double SimDT, double MJD) {

	if(launch_status == 1)
	{
		timer = timer + SimDT;
		if(timer > 300) 
		{
			oapiDeleteVessel(GetHandle());
			return;
		}
	}

	// next block of code
	ENGINESTATUS es;
	es.main = 0;
	
	OBJHANDLE hDelta_A = GetAttachmentStatus(hAtt_A);
	OBJHANDLE hDelta_B = GetAttachmentStatus(hAtt_B);

	///////////// PadA 
	if(hDelta_A)
	{
	if (launch_status == 1)launch_status = 0;

	if(oapiGetFuelMass (hDelta_A) > 0) // if its a rocket
		{
		if(vent_status == 0) SetThrusterLevel(th_hovera, 1);
		else SetThrusterLevel(th_hovera, 0);

		oapiGetEngineStatus(hDelta_A, &es);
		if(es.main > 0)SetThrusterLevel(th_steama, 1);
		else SetThrusterLevel(th_steama, 0);

		if(es.main > 0.9 && ant_status == 0)
		{
			ant_status = 1;
			launch_status = 1;
		}
		if(es.main > 0.95) DetachChild(hAtt_A);
		}
	else 
		{
		SetThrusterLevel(th_hovera, 0);
		SetThrusterLevel(th_steama, 0);
		}

	if (ant_status != 0)ant_status = 3;
	
	}
	else
	{
	SetThrusterLevel(th_steama, 0);
	SetThrusterLevel(th_hovera, 0);
	if (ant_status != 2)ant_status = 1;
	}
	
	/////// Pad B
	if(hDelta_B)
	{
	if (launch_status == 1)launch_status = 0;
	if(oapiGetFuelMass (hDelta_B) > 0)
		{
		if(vent_status == 0) SetThrusterLevel(th_hover, 1);
		else SetThrusterLevel(th_hover, 0);

		oapiGetEngineStatus(hDelta_B, &es);
		if(es.main > 0)SetThrusterLevel(th_steam, 1);
		else SetThrusterLevel(th_steam, 0);

		if(es.main > 0.9 && ant_statusb == 0)
		{
			ant_statusb = 1;
			launch_status = 1;
		}
		if(es.main > 0.95) DetachChild(hAtt_B);
		}
	else
		{
		SetThrusterLevel(th_hover, 0);
		SetThrusterLevel(th_steam, 0);
		}

	if (ant_statusb != 0)ant_statusb = 3;
	}
	else
	{
	SetThrusterLevel(th_steam, 0);
	SetThrusterLevel(th_hover, 0);
	if (ant_statusb != 2)ant_statusb = 1;
	}
	///// time accel check for vents
	if(oapiGetTimeAcceleration() > 10)
	{
	SetThrusterLevel(th_steam, 0);
	SetThrusterLevel(th_hover, 0);
	SetThrusterLevel(th_steama, 0);
	SetThrusterLevel(th_hovera, 0);
	}
	

	//////// lighting on/off check
	VECTOR3 rsuninv;
	VECTOR3 rsunloc;
	VECTOR3 rsun;
	GetGlobalPos(rsun);
	rsuninv = _V(-rsun.x, -rsun.y, -rsun.z);
	Global2Local(rsuninv,rsunloc);

	if (slca1_light->IsActive()== false && rsunloc.y < 0)
			{
			slca1_light->Activate(true);
			slca2_light->Activate(true);
			slcb1_light->Activate(true);
			slcb2_light->Activate(true);
			}
	else if (slca1_light->IsActive()== true && rsunloc.y > 0)
			{
			slca1_light->Activate(false);
			slca2_light->Activate(false);
			slcb1_light->Activate(false);
			slcb2_light->Activate(false);
			}

	if (light_status == 1)
			{
			slca1_light->Activate(false);
			slca2_light->Activate(false);
			slcb1_light->Activate(false);
			slcb2_light->Activate(false);
			}

	//////////// animations ////////////////////////////////////
	////////////////////////////////////////////////////////////

	//////////// pad a UMBS
	if (ant_status == 0) ant_proc = 0;
	else if (ant_status == 1)
	{
	double da = SimDT*ANTENNA_OPERATING_SPEED;
	ant_proc = ant_proc + da;
	if (ant_proc > 1)
		{
		ant_proc = 1;
		ant_status = 2;
		}

	}
	else if (ant_status == 2) ant_proc = 1;
	else if (ant_status == 3)
	{
	double da = SimDT*ANTENNA_OPERATING_SPEED;
	ant_proc = ant_proc - da;
	if (ant_proc < 0)
		{
		ant_proc = 0;
		ant_status = 0;
		}

	}

	/////// PAD A MSS
	if (chute_status == 0) chute_proc = 0;
	else if (chute_status == 1)
	{
	double da = SimDT*CHUTE_OPERATING_SPEED;
	chute_proc = chute_proc + da;
	if (chute_proc > 1)
		{
		chute_proc = 1;
		chute_status = 2;
		}

	}
	else if (chute_status == 2) chute_proc = 1;
	else if (chute_status == 3)
	{
	double da = SimDT*CHUTE_OPERATING_SPEED;
	chute_proc = chute_proc - da;
	if (chute_proc < 0)
		{
		chute_proc = 0;
		chute_status = 0;
		}

	}



	//////////// pad B UMBS
	if (ant_statusb == 0) ant_procb = 0;
	else if (ant_statusb == 1)
	{
	double da = SimDT*ANTENNA_OPERATING_SPEED;
	ant_procb = ant_procb + da;
	if (ant_procb > 1)
		{
		ant_procb = 1;
		ant_statusb = 2;
		}

	}
	else if (ant_statusb == 2) ant_procb = 1;
	else if (ant_statusb == 3)
	{
	double da = SimDT*ANTENNA_OPERATING_SPEED;
	ant_procb = ant_procb - da;
	if (ant_procb < 0)
		{
		ant_procb = 0;
		ant_statusb = 0;
		}

	}

	////// PAD B MSS
	if (chute_statusb == 0) chute_procb = 0;
	else if (chute_statusb == 1)
	{
	double da = SimDT*CHUTE_OPERATING_SPEED;
	chute_procb = chute_procb + da;
	if (chute_procb > 1)
		{
		chute_procb = 1;
		chute_statusb = 2;
		}

	}
	else if (chute_statusb == 2) chute_procb = 1;
	else if (chute_statusb == 3)
	{
	double da = SimDT*CHUTE_OPERATING_SPEED;
	chute_procb = chute_procb - da;
	if (chute_procb < 0)
		{
		chute_procb = 0;
		chute_statusb = 0;
		}

	}

	SetAnimation (anim_ant, ant_proc);
	SetAnimation (anim_chute, chute_proc);
	SetAnimation (anim_antb, ant_procb);
	SetAnimation (anim_chuteb, chute_procb);

}

//void slc17::clbkPostStep (double simt, double simdt, double mjd)
//{
//	
//}